* Diamonds 2.0 for the HP48 (S, SX, G, or GX)
* ONLY supported entry points are used.
*
* By: Douglas R. Cannon
* On: August 7, 1993
* Slightly modified on October 30, 1995
*
* email: dougc@erudite.com
* hp48 page: http://www.erudite.com/~dougc/hp48.htm
*
* Home address as of Sept 1996:
* 350 N. 850 E.
* American Fork, UT 84003
* (801) 763-8798
*
* Each field is separated by one Tab.  I have found that Courier New
* 8 pt font works well with tabs stops set at 1.0, 1.875, and 3 inches.
*
* Special thanks to Paul Hart for converting the source code into
* SASM format.  The source was originally written for my own assembler,
* SATASM.EXE, and has now been converted to be SASM compatible.  Paul spent
* many hours working on it, and he did a great job!  Thanks Paul!
*
* Special thanks also to Jim Donnelly for being such a cool guy.  I owe
* him thanks as well for his latest book (_An Introduction to HP 48 System
* RPL and Assembly Language Programming_), his earlier books, and miscellaneous
* help and encouragement over the last couple years.  You can learn a few
* things from reading this source code, but if you really want to learn
* something, then get Jim's book!
*
* ----------------------------------------------
* | Global Label prefix assignments:
* |
* | be: BONUS_ERASE
* | bo: BOUNCE
* | cb: CHECK_BOUNCE
* | cc: Convert Coords (ccCONVERT)
* | cf: Clear Field, or Draw logo (cfCLEAR, cfPUT_LOGO)
* | ch: Check High Scores (chCHECK_HIGH)
* | cu: Cursor (cuCURSOR)
* | db: DRAW_BALL
* | de: DRAW_LEVEL
* | dh: Display High Scores (dhHIGH)
* | dl: DRAW_LIVES
* | dm: dmDRAW_MENU
* | ds: DRAW_SCREEN
* | em: ERASE MODE
* | er: ERASE
* | fd: Font Drawer (fdFONT)
* | gr: GRAPHICS displays
* | hb: HIT_BRICK
* | il: Input Letter (inINPUT_LET)
* | in: Input Number (inINPUT_NUM)
* | is: Init Screen (isINIT_SCN)
* | la: Letter Address (laLETTERS)
* | ma: MAIN
* | me: MAIN_EVENT
* | mn: Menu (mnMENU)
* | mv: MOVE
* | na: Numbers Address (naNUMBERS)
* | nd: Number Draw (ndNUMBER)
* | pk: PAUSE_KEY
* | sc: Score (scADD_SCORE, scDISP_SCORE)
* | sd: DISOLVE (screen disolver)
* | sl: Save Logo (slSAVE_LOGO)
* | so: SOUND
* | sr: Variable Storage/Retrieval system
* | ud: Universal Delay (udDELAY)
* | wa: Wait  (waWAIT)
* |
* ----------------------------------------------

* ----------------------------------------------
* | Global Variable assignments:
* |
* | R0: Permanent storage for address of sr data.
* | R1:
* | R2:
* | R3:
* | R4:
* |-----------------
* | (by Var#)
* |
* | 0:  (PERM) current X coord
* | 1:  (PERM) current Y coord
* | 2:  (PERM) current direction of travel
* | 3:  temp storage for screen pos (mv, fd)
* | 4:  temp in (ds, db, ud, fd, nd)
* | 5:  (PERM) @ of brick graphics
* | 6:  (PERM) @ of current screen data (brick matrix)
* | 7:  temp holder, ball addr (mv, fd)
* | 8:  Permanent address of STACK bitmap area
* | 9:  Permanent address of PICT bitmap area
* | 10: |
* | 11: | 10-14 are used by cbCHK_BOUNCE
* | 12: | (no need for srRCL10_C or srSTO10_C etc.
* | 13: | cbCHK_BOUNCE calls srRCL_C etc.
* | 14: | directly.
* | 15: (PERM) current erase mode (which bricks get erased)
* | 16: (PERM) number of bricks left (not counting diamonds)
* | 17: (PERM) number of diamonds left. (done when == 0)
* | 18: (PERM) DELAY (speed)
* | 19: (PERM) Total number of levels
* | 20: (PERM) Current level number
* | 21: (PERM) @ of level 1 (X,Y,DIR)
* | 22: (PERM) # of lives left
* | 23: (PERM) Score
* | 24: Temp in (gr, ud, fd, nd)
* | 25: (PERM) time left to erase another bonus
* | 26: (PERM) next bonus life at this score
* | 27: temp in (ds, ud, fd, nd)
* | 28:
* | 29: Temp in (fd, nd)
* | 30: Temp in (fd, nd)
* | 31: Temp in (nd, in)
* | 32: (PERM) high score data @ on stack
* | 33: (PERM) practice level # (FFFFF if not in practice mode)
* | 34: Temp in (in, il, dh)
* | 35: Temp in (in, il)
* | 36: Temp in (cc, mn)
* | 37: Temp in (dh, mn)
* | 38: Temp in (dh, mn)
* | 39: (Perm) in cu
* | 40: (Perm) in cu
* | 41: (PERM) Speed toggle var
* | 42: Temp in (mn)
* | 43: Myth
* | 44: (Perm) DMID bitmap address
* | 45: (Perm) DCLEAR bitmap address
* | 46: mn
* | 47: (perm) ticks (me, ud)
* ----------------------------------------------
* | Status flags:
* |--------------------
* |
* | 0:  (temp) used for bouncing, key check (cb, mn, ud)
* | 1:  (perm) used for bouncing (me, cb)
* | 2:  (perm) used for bouncing (me, cb)
* | 3:  (perm) key move direction (me, hb)
* | 4:  (perm) 0==alive 1==DEAD (hb, me, cb)
* | 5:  (perm) 0==play 1==You WON! (hb, me)
* | 6:  (perm) 0==reg bricks 1==diamond bricks (hb)
* | 7:  (perm) 0==no key, 1==you have a key
* | 8:
* | 9:
* | 10: (temp) used for erasing the ball (mv, er, mn)
* | 11: (perm) 0 == Sound on, 1 == Sound off.
* ----------------------------------------------

ASSEMBLE

* ----------------------------------------------
* Addresses
*
BUSY	EQU	#0010C	SX/GX
CRC	EQU	#00104

* ----------------------------------------------
* Global constants
*
SLOW_SPD	EQU	#000C0	SLOW speed
NORMAL_SPD	EQU	#00080	NORMAL speed (delay in main loop)
FAST_SPD	EQU	#00040	FAST speed
SUPER_SPD	EQU	#00000	SUPER speed
SPEED_UP	EQU	#00040	amount to subtract from speed to speed up.
BONUS_TIME	EQU	#00070
BONUS_LIFE	EQU	#05000	in decimal
LIVES	EQU	#00005
NIBBLES	EQU	#0020D	#FFh + #10Eh (#FFh==51*5) (#10Eh==15*18)
VARS	EQU	#000FF	5 * (50+1) == #FFh
* -----------------------------------------------

* -------------
* | When this main object is executed, the stack
* | should look like this:
* |
* | L5: DIAMONDS LEVELS
* | L4: HIGH SCORES
* | L3: Address of ABUFF (#12655h SYSEVAL)
* | L2: A STRING (with a length of at least NIBBLES + #77F)
* | L1: A STRING (with a 1 CHR or 0 CHR depending on flag -56)
* |
* -------------

	NIBASC	/HPHP48-L/
RPL
::
ASSEMBLE
	CON(5)	=DOEXT0
	REL(5)	LEVELS_END
	NIBHEX	23000	Number of levels (50)
	NIBHEX	70202	Level 1
	NIBHEX	0000000000
	NIBHEX	0091001B00
	NIBHEX	6A800008A6
	NIBHEX	8A802208A8
	NIBHEX	8A802208A8
	NIBHEX	6A800008A6
	NIBHEX	00B1001900
	NIBHEX	0000000000
	NIBHEX	73831	Level 2
	NIBHEX	0F22662290
	NIBHEX	00EEEEEE00
	NIBHEX	0008888000
	NIBHEX	0000110000
	NIBHEX	0000110000
	NIBHEX	0008888000
	NIBHEX	00EEEEEE00
	NIBHEX	09226622F0
	NIBHEX	73C21	Level 3
	NIBHEX	91000000E4
	NIBHEX	B1000000EE
	NIBHEX	23333000C4
	NIBHEX	22255000CC
	NIBHEX	22255000A4
	NIBHEX	23333000AA
	NIBHEX	D100000084
	NIBHEX	F100000088
	NIBHEX	73021	Level 4
	NIBHEX	7000000007
	NIBHEX	B10000001C
	NIBHEX	AD100001AC
	NIBHEX	AC220022AC
	NIBHEX	AC220022AC
	NIBHEX	AC100001BC
	NIBHEX	A10000001D
	NIBHEX	7000000007
	NIBHEX	B5831	Level 5
	NIBHEX	2250E364F4
	NIBHEX	2230E30000
	NIBHEX	2230E30000
	NIBHEX	3330E30888
	NIBHEX	0000E30CCC
	NIBHEX	1100E30111
	NIBHEX	9100E30100
	NIBHEX	D100050100
	NIBHEX	31831	Level 6
	NIBHEX	0010000100
	NIBHEX	00A0000E00
	NIBHEX	1AA6BF6EE1
	NIBHEX	0002222000
	NIBHEX	0002222000
	NIBHEX	5EE6FB6AA4
	NIBHEX	00E0000A00
	NIBHEX	0010000100
	NIBHEX	73802	Level 7
	NIBHEX	EE002200EE
	NIBHEX	06A0000A60
	NIBHEX	00B1001F00
	NIBHEX	2000000002
	NIBHEX	2000000002
	NIBHEX	00F1001B00
	NIBHEX	06E0000E60
	NIBHEX	AA002200AA
	NIBHEX	B5622	Level 8
	NIBHEX	BA00EE0001
	NIBHEX	5700E90001
	NIBHEX	0000EE0001
	NIBHEX	2323200006
	NIBHEX	3232300006
	NIBHEX	0000880001
	NIBHEX	47008F0001
	NIBHEX	AA00880001
	NIBHEX	34831	Level 9
	NIBHEX	6200000026
	NIBHEX	270A11E072
	NIBHEX	000A9D5000
	NIBHEX	01AA00EE10
	NIBHEX	018800CC10
	NIBHEX	0004BFC000
	NIBHEX	270811C072
	NIBHEX	6200000026
	NIBHEX	B5831	Level 10
	NIBHEX	0000026CC0
	NIBHEX	00002A8C89
	NIBHEX	0002A8C8C1
	NIBHEX	002A8C8C10
	NIBHEX	02A6C8C100
	NIBHEX	2A8C8C1000
	NIBHEX	BCC8C10000
	NIBHEX	0CD7100000
	NIBHEX	B5E02	Level 11
	NIBHEX	6100000016
	NIBHEX	1000220001
	NIBHEX	0009EEE000
	NIBHEX	002E008200
	NIBHEX	002E008200
	NIBHEX	000888F000
	NIBHEX	1000220001
	NIBHEX	6100000016
	NIBHEX	B5622	Level 12
	NIBHEX	1000150001
	NIBHEX	0000F50000
	NIBHEX	00005A0000
	NIBHEX	1EE4221898
	NIBHEX	5EEE224148
	NIBHEX	00001A0000
	NIBHEX	0000B40000
	NIBHEX	1000AA0001
	NIBHEX	70831	Level 13
	NIBHEX	0000000000
	NIBHEX	0000220000
	NIBHEX	0001FE1000
	NIBHEX	001DCCC100
	NIBHEX	01BAAAAA10
	NIBHEX	69EEE88887
	NIBHEX	6ECA00ACE6
	NIBHEX	0220000220
	NIBHEX	B5831	Level 14
	NIBHEX	00002080C0
	NIBHEX	000001080C
	NIBHEX	6000001080
	NIBHEX	0900000108
	NIBHEX	C0100000D0
	NIBHEX	0C01000006
	NIBHEX	80C0100000
	NIBHEX	080C020000
	NIBHEX	B2021	Level 15
	NIBHEX	6EE0000AA6
	NIBHEX	EE002200AA
	NIBHEX	E00911D00A
	NIBHEX	0022001100
	NIBHEX	0011002200
	NIBHEX	C00B11F008
	NIBHEX	CC00220088
	NIBHEX	6CC0000886
	NIBHEX	B5831	Level 16
	NIBHEX	222C00088F
	NIBHEX	2223000666
	NIBHEX	3333000AAA
	NIBHEX	EEEE0CCCC9
	NIBHEX	B88E0C0000
	NIBHEX	888E0C0111
	NIBHEX	EEEE0C0100
	NIBHEX	666E0D0100
	NIBHEX	73621	Level 17
	NIBHEX	600E000E06
	NIBHEX	000E000E07
	NIBHEX	333F111D0A
	NIBHEX	222200020A
	NIBHEX	222200020A
	NIBHEX	222D111F0A
	NIBHEX	005C000C0A
	NIBHEX	B05C000C46
	NIBHEX	F1021	Level 18
	NIBHEX	EE009D00AA
	NIBHEX	E00222200A
	NIBHEX	E00100100A
	NIBHEX	0070000600
	NIBHEX	0060000700
	NIBHEX	C001001008
	NIBHEX	C002222008
	NIBHEX	CC00BF0088
	NIBHEX	B2E02	Level 19
	NIBHEX	0882112EE0
	NIBHEX	EEE0000888
	NIBHEX	29000000F2
	NIBHEX	1100000016
	NIBHEX	6100000011
	NIBHEX	2F00000092
	NIBHEX	8880000EEE
	NIBHEX	0EE2112880
	NIBHEX	80721	Level 20
	NIBHEX	00A0000C00
	NIBHEX	0AA6007CC0
	NIBHEX	2004004002
	NIBHEX	20AB11DC02
	NIBHEX	20AB11DC02
	NIBHEX	2005005002
	NIBHEX	0AA7006CC0
	NIBHEX	00A0000C00
	NIBHEX	F1202	Level 21
	NIBHEX	88088CC0CC
	NIBHEX	820F22B02C
	NIBHEX	0000000000
	NIBHEX	810067001C
	NIBHEX	A10076001E
	NIBHEX	0000000000
	NIBHEX	A20D22902E
	NIBHEX	AA0AAEE0EE
	NIBHEX	B5831	Level 22
	NIBHEX	6000000006
	NIBHEX	7000190007
	NIBHEX	6000820006
	NIBHEX	7000180007
	NIBHEX	6000820006
	NIBHEX	7000180007
	NIBHEX	6000820006
	NIBHEX	7000000000
	NIBHEX	73A12	Level 23
	NIBHEX	0078008700
	NIBHEX	68009C00C6
	NIBHEX	00EE00CE00
	NIBHEX	6C00D100C6
	NIBHEX	00E1001C00
	NIBHEX	6E001F0026
	NIBHEX	007E007800
	NIBHEX	6200220026
	NIBHEX	B5831	Level 24
	NIBHEX	2628F86262
	NIBHEX	0000000000
	NIBHEX	176ADA3333
	NIBHEX	0000000000
	NIBHEX	3333CBC671
	NIBHEX	0000000000
	NIBHEX	176E9E3333
	NIBHEX	0000000000
	NIBHEX	73E01	Level 25
	NIBHEX	1EE00008FC
	NIBHEX	0185008DC0
	NIBHEX	0016226C00
	NIBHEX	0002222000
	NIBHEX	0002222000
	NIBHEX	0016226800
	NIBHEX	01AA004880
	NIBHEX	1BA0000988
	NIBHEX	70412	Level 26
	NIBHEX	0EB00001A2
	NIBHEX	0E10000FA2
	NIBHEX	0E600001A2
	NIBHEX	0E100006A2
	NIBHEX	0EB00001A2
	NIBHEX	0E10000FA2
	NIBHEX	0E600001A2
	NIBHEX	0E100006A2
	NIBHEX	F4C21	Level 27
	NIBHEX	BD610700A2
	NIBHEX	11E00C00A2
	NIBHEX	00E00C00A2
	NIBHEX	00E00C00A2
	NIBHEX	00E00C00A2
	NIBHEX	00E00C00A2
	NIBHEX	00E00C00A2
	NIBHEX	1070016FA2
	NIBHEX	70411	Level 28
	NIBHEX	100072F008
	NIBHEX	700062900E
	NIBHEX	600072F008
	NIBHEX	700062900E
	NIBHEX	600072F008
	NIBHEX	700062900E
	NIBHEX	600072F008
	NIBHEX	7000000006
	NIBHEX	34202	Level 29
	NIBHEX	000A630003
	NIBHEX	030030030A
	NIBHEX	0030703A3A
	NIBHEX	322363AAAA
	NIBHEX	B223631111
	NIBHEX	0330703131
	NIBHEX	0300300301
	NIBHEX	0001630003
	NIBHEX	70202	Level 30
	NIBHEX	000800129B
	NIBHEX	0008000244
	NIBHEX	0015000211
	NIBHEX	4888000200
	NIBHEX	0018000200
	NIBHEX	000BA5AA5A
	NIBHEX	000A000001
	NIBHEX	0005000014
	NIBHEX	B5831	Level 31
	NIBHEX	0000000000
	NIBHEX	0011111100
	NIBHEX	0086AA6A00
	NIBHEX	008A228A00
	NIBHEX	008A228A00
	NIBHEX	0086886A00
	NIBHEX	0091111B00
	NIBHEX	0000000000
	NIBHEX	B5231	Level 32
	NIBHEX	2005001004
	NIBHEX	2003001006
	NIBHEX	2006001003
	NIBHEX	2003001006
	NIBHEX	2006001003
	NIBHEX	2003001006
	NIBHEX	2006000003
	NIBHEX	2003000006
	NIBHEX	31202	Level 33
	NIBHEX	00B1111100
	NIBHEX	00EE1AAA00
	NIBHEX	00111D1F00
	NIBHEX	00E1221100
	NIBHEX	00E1221A00
	NIBHEX	00E1111198
	NIBHEX	001AECAE88
	NIBHEX	001AECAE88
	NIBHEX	F1412	Level 34
	NIBHEX	002EEAA200
	NIBHEX	0E91001DA0
	NIBHEX	EE100001AA
	NIBHEX	2100670012
	NIBHEX	2100760012
	NIBHEX	CC10000188
	NIBHEX	0CB1001F80
	NIBHEX	002CC88200
	NIBHEX	F1231	Level 35
	NIBHEX	00F1006B00
	NIBHEX	A02600120E
	NIBHEX	C000000008
	NIBHEX	A008EAC00E
	NIBHEX	A00AC8E00E
	NIBHEX	C000000008
	NIBHEX	A02100620E
	NIBHEX	00D6001900
	NIBHEX	05D21	Level 36
	NIBHEX	0000F80022
	NIBHEX	E100180022
	NIBHEX	E900001800
	NIBHEX	00E6007800
	NIBHEX	00E7006800
	NIBHEX	00E10000F8
	NIBHEX	2200E10018
	NIBHEX	2200E90000
	NIBHEX	31622	Level 37
	NIBHEX	6010000102
	NIBHEX	20C0110C02
	NIBHEX	20A0AA0A02
	NIBHEX	6000AD0007
	NIBHEX	7000BC0006
	NIBHEX	20A0CC0A02
	NIBHEX	20C0110C02
	NIBHEX	2010000106
	NIBHEX	F1831	Level 38
	NIBHEX	2C00B800EF
	NIBHEX	C400850041
	NIBHEX	00E1002E00
	NIBHEX	00E200C100
	NIBHEX	6E00A20046
	NIBHEX	58002A0088
	NIBHEX	00C5004A00
	NIBHEX	00D1009500
	NIBHEX	70021	Level 39
	NIBHEX	0EE800CAA0
	NIBHEX	01E800CA10
	NIBHEX	0000000000
	NIBHEX	29060010D2
	NIBHEX	2B010060F2
	NIBHEX	0000000000
	NIBHEX	01CA00E810
	NIBHEX	0CCA00E880
	NIBHEX	83802	Level 40
	NIBHEX	E000A00020
	NIBHEX	0501090C0A
	NIBHEX	00E000A000
	NIBHEX	0106010F0C
	NIBHEX	2000200080
	NIBHEX	06010D0106
	NIBHEX	0080008000
	NIBHEX	020B040601
	NIBHEX	B5202	Level 41
	NIBHEX	151122E000
	NIBHEX	188F22E000
	NIBHEX	188422E000
	NIBHEX	188122E000
	NIBHEX	5511119000
	NIBHEX	48EEEEE000
	NIBHEX	0000000001
	NIBHEX	0000000014
	NIBHEX	44622	Level 42
	NIBHEX	0200A00200
	NIBHEX	2005004060
	NIBHEX	00100A0002
	NIBHEX	0500F00B00
	NIBHEX	A00A00E010
	NIBHEX	00600E000E
	NIBHEX	10040010E0
	NIBHEX	0200700E00
	NIBHEX	15331	Level 43
	NIBHEX	100E00E002
	NIBHEX	06001800B0
	NIBHEX	200C001002
	NIBHEX	0D00A10060
	NIBHEX	200100A002
	NIBHEX	06001C0090
	NIBHEX	2008008001
	NIBHEX	0F00E10060
	NIBHEX	70831	Level 44
	NIBHEX	2201005000
	NIBHEX	20100A0006
	NIBHEX	01006000A0
	NIBHEX	100A000B00
	NIBHEX	00B000A001
	NIBHEX	0A00060010
	NIBHEX	6000A00102
	NIBHEX	0004001022
	NIBHEX	F4411	Level 45
	NIBHEX	0CC0CE0000
	NIBHEX	0E20CE0000
	NIBHEX	02D0110670
	NIBHEX	0120110C00
	NIBHEX	00E0CE0210
	NIBHEX	0760CE0C20
	NIBHEX	0000CE02F0
	NIBHEX	0000110EE0
	NIBHEX	93202	Level 46
	NIBHEX	2000AA0002
	NIBHEX	FE00C100AA
	NIBHEX	0450C60140
	NIBHEX	00E2AA2C00
	NIBHEX	00EE4EEC00
	NIBHEX	0CC0110D70
	NIBHEX	1500BE0015
	NIBHEX	20006C0002
	NIBHEX	70831	Level 47
	NIBHEX	00BE00EE00
	NIBHEX	00DA00AC00
	NIBHEX	00FA00CA00
	NIBHEX	0016006100
	NIBHEX	6100A70016
	NIBHEX	EC00AA00CE
	NIBHEX	EC002200CE
	NIBHEX	2200110022
	NIBHEX	70831	Level 48
	NIBHEX	2100600100
	NIBHEX	2600100600
	NIBHEX	2100ECAECA
	NIBHEX	2100D00100
	NIBHEX	2B00100F00
	NIBHEX	2ECAECA100
	NIBHEX	2600100600
	NIBHEX	2100600100
	NIBHEX	73232	Level 49
	NIBHEX	81001800CA
	NIBHEX	8F004500DA
	NIBHEX	5500660041
	NIBHEX	0022002200
	NIBHEX	0022002200
	NIBHEX	5400660041
	NIBHEX	CB0045009E
	NIBHEX	C1001C00EE
	NIBHEX	31231	Level 50
	NIBHEX	C8C8C8C8C8
	NIBHEX	800D00100C
	NIBHEX	C001009008
	NIBHEX	111222600C
	NIBHEX	1006222EAE
	NIBHEX	600100B001
	NIBHEX	100F001001
	NIBHEX	EAEAEAEAEA
LEVELS_END
RPL
	'
	ID DIAM.HIGH
	@
	?SKIP
	::
ASSEMBLE
	CON(5)	=DOEXT0
	REL(5)	SCORES_END
	NIBHEX	00001	Top high score
	NIBHEX	02840500	" HP"
	NIBHEX	00010
	NIBHEX	02840500	" HP"
	NIBHEX	00100
	NIBHEX	02840500	" HP"
	NIBHEX	01000
	NIBHEX	02840500	" HP"
	NIBHEX	10000	Lowest high score
	NIBHEX	02840500	" HP"
SCORES_END
RPL
	TOTEMPOB
	;
	RECLAIMDISP
	TURNMENUOFF
	ABUFF
	# 264
	ONE
	MAKEGROB
	DO>STR
	FIFTYSIX
	TestSysFlag
CODE
	P=	0
	GOSBVL	=SAVPTR

	INTOFF
	ST=0	15	STOP those interrupts

	D0=(5)	BUSY	some annunciators
	C=DAT0	1	read the values
	CBIT=0	0	turn off BUSY annunciator
	DAT0=C	1	write it

maLEVEL1	A=DAT1	A	object in level 1 (TRUE/FALSE)
	LC(5)	=FALSE
	?A=C	A	is sysflag -56 clear (sound on)
	GOYES	maSOUND_ON	set sound on flag
	ST=1	11	set sound off
	GOTO	maLEVEL2	go do level 2
maSOUND_ON	ST=0	11	set sound on

maLEVEL2	D1=D1+	5	move to stack level 2
	A=DAT1	A	addr of object in level 2 (data string)
	A=A+CON	A,10	point to string data portion
	R0=A.F	A	save it here forever

	LC(5)	NIBBLES	offset past data portion
	C=C+A	A	add addr
	GOSUBL	srSTO8_C	permanent storage

maLEVEL3	D1=D1+	5	move to stack level 3
	A=DAT1	A	get address of ABUFF
	LCHEX	#00014	skip prolog, length, etc (20 DEC)
	C=C+A	A
	GOSUBL	srSTO9_C	permanent storage

maLEVEL4	D1=D1+	5	move to stack level 4
	C=DAT1	A	addr of object in level 4 (high scores)
	C=C+CON	A,10	point to data portion
	GOSUBL	srSTO32_C	permanent storage

maLEVEL5	D1=D1+	5	move to stack level 5
	A=DAT1	A	addr of object in level 5 (levels)
	A=A+CON	A,10	point to # of levels
	D0=A
	C=DAT0	A	get number of levels
	GOSUBL	srSTO19_C	store it for good
	A=A+CON	A,5	now pointing to level 1 (to X,Y,DIR)
	C=A	A	must put it here
	GOSUBL	srSTO21_C	permanent storage

	LCHEX	#00050	|
	GOSUBL	srSTO39_C	|
	C=0	A	|
	GOSUBL	srSTO40_C	| init for cuCURSOR

	GOSUBL	isINIT_SCN	draw screen (except for inside of play field)

	LC(5)	NORMAL_SPD	"NORMAL SPEED"
	GOSUBL	srSTO41_C	init speed var


* This is the menu routine.  It displays the menu, and
* handles the key presses.

mnMENU	GOSUBL	cfCLEAR	clear playing field
	GOSUBL	dmDRAW_MENU	draw menu
	LCHEX	#FFFFF	this means not practice mode
	GOSUBL	srSTO33_C	put it here
	ST=0	10	no letter is bumped up yet.

mnSTOP_KEY	LCHEX	#1FF	| check here to see
	OUT=C		| if they are still pressing
	GOSBVL	=CINRTN	| a key
	LAHEX	#0003F	|
	A=A&C	A	|
	?A#0	A	|
	GOYES	mnYES_KEY	|
	ST=0	0	no key being pressed
	GOTO	mnOK_KEY
mnYES_KEY	ST=1	0	a key is being pressed

mnOK_KEY	C=0	A
	GOSUBL	srSTO43_C	stop spreading rumors

mnANIMATE	C=PC
	GOTO	mnA_NEXT1

	NIBHEX	0000600000000000000000
	NIBHEX	0FF090C1E8100C3CC700F0
	NIBHEX	C0098039152E3242181E01
	NIBHEX	4009880A0521C24210A102
	NIBHEX	400A840A05A0014110A032
	NIBHEX	4814844A02200149004CC1
	NIBHEX	446482AA00281149064200
	NIBHEX	8488829400242149094200
	NIBHEX	8488888400222188884E10
	NIBHEX	84488844002221088840E3
	NIBHEX	8428803400222104944004
	NIBHEX	8818400240249004134F04
	NIBHEX	80044832A2488004102014
	NIBHEX	800B4442A540801410B312
	NIBHEX	80884242A54049221C4C02
	NIBHEX	0978A242A441492A034001
	NIBHEX	0708338D183E37C9F08FF0

mnA_NEXT1	C=C+CON	A,4
	GOSUBL	srSTO36_C	DCLEAR bitmap

	C=PC
	GOTO	mnA_NEXT2

	NIBHEX	0000000000000000000000
	NIBHEX	0000600000000000000000
	NIBHEX	0FF070C0E8100C3CC700F0
	NIBHEX	8FF070F1F81E3C3CCF1EF1
	NIBHEX	8FF178F1F81FFC3ECF1FC1
	NIBHEX	87E378B1FD9FFC36EF3300
	NIBHEX	83837C11FF97EC36E93100
	NIBHEX	03076C03FF93CC36E03100
	NIBHEX	03076603FF91CC77603100
	NIBHEX	03876683FF91CCF7603F10
	NIBHEX	03C76EC3FF91CCF3683FF3
	NIBHEX	07E72FF9BF936EF3EC30F3
	NIBHEX	0FF337C91D377EF3EF10E3
	NIBHEX	0FF03389183F7EE3EF00E1
	NIBHEX	0F703189183F36C1E383F1
	NIBHEX	06001189183E36C1F08FF0
	NIBHEX	0000000000000000000000

mnA_NEXT2	C=C+CON	A,4
	GOSUBL	srSTO37_C	DMID bitmap

	GOSUBL	srRCL9_C	get ABUFF bitmap address
	LAHEX	00068	offset
	C=C+A	A
	GOSUBL	srSTO38_C

mnA_TOP	B=0	A
	LCHEX	#00011
	GOSUBL	srSTO42_C	counter
	GOSUBL	srRCL38_C	offset
	D0=C

mnA_LINE	LCHEX	#00004
	D=C	A

mnA_BLOCK	A=DAT0	4	usually random (from screen)
	D1=(5)	CRC	CRC
	A=DAT1	4	a really random number
	C=DAT0	4
	C=DAT1	4	another random number
	A=A!C	A	Make it a little darker

	GOSUBL	srRCL37_C	DMID bitmap
	C=C+B	A	add nibble offset from line
	D1=C
	C=DAT1	4	Mask
	A=A&C	A	Mask off from random nibbles
	GOSUBL	srRCL36_C	DCLEAR bitmap
	C=C+B	A	add nibble offset
	D1=C
	C=DAT1	4	outline
	A=A!C	A	OR it on
	DAT0=A	4	write to screen

	D0=D0+	4
	B=B+CON	A,4	increment nibble offset
	D=D-1	A
	GONC	mnA_BLOCK

	A=DAT0	2	random from screen
	D1=(5)	CRC	CRC
	A=DAT1	2	really random
	C=DAT0	2
	C=DAT1	2	really random
	A=A!C	A	make it darker

	GOSUBL	srRCL37_C	DMID bitmap
	C=C+B	A	add nibble offset from line
	D1=C
	C=DAT1	2	mask
	A=A&C	A	mask off
	GOSUBL	srRCL36_C	DCLEAR bitmap
	C=C+B	A	nibble offset
	D1=C
	C=DAT1	2	outline
	A=A!C	A	OR it on
	DAT0=A	2	write to screen
	B=B+CON	A,2	increment nibble offset

	GOSUBL	srRCL42_C	line counter
	C=C-1	A
	GOSUBL	srSTO42_C
	?C=0	A	are we done yet?
	GOYES	mnA_KEYS
	D0=D0+	14	point to next line
	GOTO	mnA_LINE	do next line

mnA_KEYS	?ST=0	0	is no key being still pressed?
	GOYES	mnA_NOKEY
	GOTO	mnSTOP_KEY
mnA_NOKEY	GOTO	mnCHECK_KEY	check for (A-E)

mnA_BUMP	?ST=0	10	no need to bump down?
	GOYES	mnA_BCHECK	OK, lets see if we bump up
	GOSUBL	srRCL44_C	counter
	C=C-1	A	dec counter
	GOSUBL	srSTO44_C	store it
	?C=0	A	are we done waiting?
	GOYES	mnA_TODN	yes? go down
	GOTO	mnA_TOP	nope... go to top again
mnA_TODN	ST=0	10	set flag
	GOTO	mnA_DOWN	bump me down

mnA_BCHECK	D1=(5)	CRC	CRC
	A=0	A
	A=DAT1	2	random
	LCHEX	#00010	a probability
	?A<=C	A	did we get a number less?
	GOYES	mnA_STBUMP	go start the bump
	GOTO	mnA_TOP	otherwise, go to TOP

mnA_STBUMP	D0=(5)	=TIMER2	TIMER2
	D1=(5)	CRC	CRC
	A=DAT0	8	read 8 nibbles
	A=DAT0	8	read 8 more
	A=0	W	clear out all of A
	A=DAT1	2	a random number
	C=0	W	clear out all of C
	LCHEX	#1B	a number
	GOSUBL	DIV	divide them

mnA_CHOOSE		C=PC
	GOTO	mnA_BNEXT

	NIBHEX	00F30
	NIBHEX	108F1
	NIBHEX	30CF0
	NIBHEX	50E70
	NIBHEX	80CF0
	NIBHEX	A0E70
	NIBHEX	C0F30
	NIBHEX	D08F1
	NIBHEX	F0CF0
	NIBHEX	11E70

mnA_BNEXT	C=C-1	A	5 before the data
	D0=C
mnA_CHECK	D0=D0+	5
	A=A-1	A
	GONC	mnA_CHECK

	GOSUBL	srRCL9_C	get ABUFF bitmap
	LAHEX	#002F1	offset to my name
	C=C+A	A	add the offset
	A=0	A	clear A
	A=DAT0	2	read 2 data nibbles
	C=C+A	A	offset to correct letter
	D0=D0+	2	increment data pointer
	A=0	A
	A=DAT0	3	mask
	D0=C		address on screen

	ST=1	10	we've bumped up!
	LCHEX	#00004	timer...counter, really
	GOSUBL	srSTO44_C	store it here

mnA_UP	B=A	A	mask
	LCHEX	#00007	count
	D=C	A

mnA_UPLINE	A=DAT0	3	read screen
	A=A&B	A	mask
	D0=D0-	16	|
	D0=D0-	16	|
	D0=D0-	2	| up one line
	C=DAT0	3	read 1 line above
	B=-B-1	A	inverse the mask
	C=C&B	A	mask
	A=A!C	A	OR on letter
	DAT0=A	3	write it to screen

	B=-B-1	A	inverse the mask again
	D0=D0+	16	|
	D0=D0+	16	|
	D0=D0+	16	|
	D0=D0+	16	|
	D0=D0+	4	| down 2 lines

	D=D-1	A	decrement counter
	GONC	mnA_UPLINE

	CD0EX
	LAHEX	#00066	three lines
	C=C-A	A	move pointer up 3 lines
	GOSUBL	srSTO45_C	store it here
	C=B	A	mask
	GOSUBL	srSTO46_C	store it here
	GOTO	mnA_TOP	go flicker the logo

mnA_DOWN	GOSUBL	srRCL46_C	mask
	B=C	A	put it here
	GOSUBL	srRCL45_C	screen pointer
	D0=C
	LCHEX	#00007	counter
	D=C	A

mnA_DNLINE	A=DAT0	A	read the screen
	A=A&B	A	mask off a letter
	D0=D0+	16	|
	D0=D0+	16	|
	D0=D0+	2	| move down 1 line
	C=DAT0	3	read 3 nibbles from screen
	B=-B-1	A	inverse the mask
	C=C&B	A	mask off stuff that stays
	A=A!C	A	OR it on
	DAT0=A	3

	B=-B-1	A	inverse the mask again
	D0=D0-	16	|
	D0=D0-	16	|
	D0=D0-	16	|
	D0=D0-	16	|
	D0=D0-	4	| move up 2 lines

	D=D-1	A	decrement counter
	GONC	mnA_DNLINE
	GOTO	mnA_TOP


mnCHECK_KEY	C=0	A	clear out C
	GOSUBL	udDELAY	check for [ON] (and [+/-])

	LCHEX	#002
	OUT=C
	GOSBVL	=CINRTN
	?CBIT=0	4	[A] key
	GOYES	mnNO_A
	GOTO	maSTART_GAME

mnNO_A	LCHEX	#010
	OUT=C
	GOSBVL	=CINRTN
	?CBIT=0	0	[<-] key
	GOYES	mnNO_BS
	GOLONG	maEND	The user just exited.

mnNO_BS	LCHEX	#100
	OUT=C
	GOSBVL	=CINRTN
	?CBIT=0	4	[B] key
	GOYES	mnNO_B
	GOLONG	dhHIGH	dhHIGH does a GOTO mnMENU

mnNO_B	?CBIT=0	2	[D] key
	GOYES	mnNO_D

mnD	GOSUBL	srRCL41_C	SPEED toggle var
	LA(5)	SPEED_UP	get speed-up var
	C=C-A	A	subtract it
	GONC	mnD_OK	no carry? we're OK
	LC(5)	SLOW_SPD	go to SLOW speed
	GOSUBL	srSTO41_C	store new speed
	GOSUBL	soMENU_DOWN	make DOWN sound
	GOTO	mnMENU

mnD_OK	GOSUBL	srSTO41_C	store new speed
	GOSUBL	soMENU_UP	make UP sound
	GOTO	mnMENU	display new menu

mnNO_D	?CBIT=0	1	[E] key
	GOYES	mnNO_E
	GOSUBL	cfCLEAR	Clear the playing field
	GOLONG	dmDRAW_HLP	Draw the Help screen graphic (auto RTN)

mnHSTOP_KEY	LCHEX	#1FF	| check here to see
	OUT=C		| if they are still pressing
	GOSBVL	=CINRTN	| a key
	LAHEX	#0003F	|
	A=A&C	A	|
	?A#0	A	|
	GOYES	mnHSTOP_KEY	|
	GOTO	mnMENU

mnNO_E	?CBIT=1	3	[C] key
	GOYES	mnYES_C
	GOTO	mnA_BUMP	back to the Diamonds animator!

mnYES_C	GOSUBL	cfCLEAR	clear the field, leave logo

	C=PC
	GOTO	mnLEVEL

	NIBHEX	54E445542502	|
	NIBHEX	6425F4D40213	|
	NIBHEX	0245F4024353	|
	NIBHEX	00	| "ENTER FROM 1 TO 45"

	NIBHEX	052514344594	]
	NIBHEX	345402	] "PRACTICE "
	NIBHEX	C4546554C402	|
	NIBHEX	00	| "LEVEL "

mnLEVEL	C=C+CON	A,4
	D0=C

	LCHEX	#00F5F	{#5h #1Eh}
	GOSUBL	fdFONT	display "ENTER FROM 1 TO 45"

	LCHEX	#013FA	{#5h #27h}
	GOSUBL	fdFONT	display "PRACTICE LEVEL "

	GOLONG	inINPUT_NUM	get a number from keyboard (into C)
mnIN_RTN	LAHEX	#0000F
	A=A&C	A
	C=C-A	A
	CSR	A
mnTENS	?C=0	A
	GOYES	mnNO_TENS
	A=A+CON	A,10
	C=C-1	A
	GOTO	mnTENS

mnNO_TENS	A=A-1	A	level 1 is really a 0
	C=A	A	put the level num here

	GOSUBL	srRCL19_A	total number of levels
	?C>=A	A	is this num too big?
	GOYES	mnL_BAD
	LAHEX	#0002D	LEVEL (45+1) is too big
	?C>=A	A	is this too big?
	GOYES	mnL_BAD

	GOSUBL	srSTO33_C	put the level here to practice
	GOTO	maSTART_GAME	this returns to mnMENU

mnL_BAD	GOTO	mnMENU	bad level #, go to menu

* This is executed only when you first start a game.

maSTART_GAME	GOSUBL	srRCL41_C	delay toggle var
	GOSUBL	srSTO18_C	save as starting speed

	LC(5)	LIVES	number of lives
	GOSUBL	srSTO22_C	store it
	LC(5)	BONUS_LIFE	points until first extra life
	GOSUBL	srSTO26_C	store it here
	C=0	A
	GOSUBL	srSTO20_C	level 1 (0+1)
	GOSUBL	srSTO23_C	score is zero
	GOSUBL	scDISP_SCORE	display the 00000 score

* This is executed at the start of each new level

maPLAY_AGAIN	GOSUBL	srRCL33_C	check if we're in practice mode
	LAHEX	#FFFFF
	?A=C	A	it's not practice mode?
	GOYES	maPA_NOTP
	GOSUBL	srSTO20_C	put practice level # here
maPA_NOTP	GOSUBL	srRCL20_C	get level #
	LAHEX	#00055	#85d nibbles per level
	GOSBVL	=MUL#	put result in B, field A
	C=B	A
	GOSUBL	srRCL21_A	pointing to level 1
	A=A+C	A	pointing to current level
	A=A+CON	A,5	pointing to brick data
	GOSUBL	srSTO6_A	save it

	GOSUBL	deDRAW_LEVEL	display level number
	GOSUBL	beBONUS_DRAW	fill the bonus area
	GOSUBL	sdDISOLVE	disolve the bricks onto the screen
	GOTO	maSET_FLAGS

* This is executed each time you die (with some lives left)

maSTART_OVER	GOSUBL	dsQUICK_DRAW	draw the bricks quickly, not disolve
maSET_FLAGS	ST=0	6	you can't knock out diamonds yet.
	ST=0	7	you have no key
	GOSUBL	hbDRAW_KEY	make sure no key is showing
	LCHEX	#00001	white brick
	GOSUBL	srSTO15_C	erase mode
	GOSUBL	emDRAW_BRICK	show erase brick
	LC(5)	BONUS_TIME	a good time...
	GOSUBL	srSTO25_C	bonus time

	GOSUBL	dlDRAW_LIVES	draw lives left
	GOSUBL	srRCL6_C	get level addr
	C=C-CON	A,5	now pointing to X,Y and DIR
	D0=C		in D0 (pointing to X)
	C=0	A
	C=DAT0	2	read X
	GOSUBL	srSTO0_C	X
	D0=D0+	2	pointing to Y
	C=DAT0	2	read Y
	GOSUBL	srSTO1_C	Y
	D0=D0+	2	pointing to DIR
	C=0	A
	C=DAT0	1	read Direction
	GOSUBL	srSTO2_C	store it

	GOSUB	grGET_READY	display the GET READY window

	GOSUBL	srRCL43_C	check for myths
	?C=0	A	none?
	GOYES	maMOVE
	LCHEX	#020
	OUT=C
	GOSBVL	=CINRTN
	?CBIT=0	3	no key?
	GOYES	ma_M1
	GOTO	maNEXT_LEVEL

ma_M1	LCHEX	#100
	OUT=C
	GOSBVL	=CINRTN
	?CBIT=0	2
	GOYES	maMOVE
	GOSUBL	srRCL20_C	current level #
ma_M3	C=C-1	A
	GONC	ma_M2
	GOSUBL	srRCL19_C
	C=C-1	A
ma_M2	GOSUBL	srSTO20_C
	GOTO	maPLAY_AGAIN


maMOVE	GOLONG	meMAIN_EVENT	instead of GOSUB (free up RSTK)
maMAIN_RTN	?ST=0	5	did you lose, or quit?
	GOYES	maCHECK_LOSE	go check

	C=0	A
	GOSUBL	srSTO2_C	set direction to none
	ST=1	10	set to erase
	GOSUBL	mvMOVE	erase the ball
	ST=0	10	set to draw
	GOSUBL	cnCONGRATS	BONUS, or NO BONUS banners

	GOSUBL	srRCL33_C	get practice indicator
	LAHEX	#FFFFF
	?A=C	A	you're not practicing?
	GOYES	maNEXT_LEVEL	go to next level
	GOTO	maPLAY_AGAIN	otherwise, go play again.

maNEXT_LEVEL	GOSUBL	srRCL20_C	get current level#
	C=C+1	A
	GOSUBL	srRCL19_A	get total number of levels
	?C<A	A	are we still OK?
	GOYES	maSET1

	GOSUBL	srRCL18_C	get current ball speed
	LA(5)	SPEED_UP	get speed-up var
	C=C-A	A	subtract it
	GONC	maOK	no carry? we're OK.
	LCHEX	#00001	make it 1 (FASTest)
maOK	GOSUBL	srSTO18_C
	C=0	A	level 0 (again...)

maSET1	GOSUBL	srSTO20_C	set new level#
	GOTO	maPLAY_AGAIN	play again!

maCHECK_LOSE	?ST=0	4	did you quit?
	GOYES	maGAME_OVER	done with game.

	GOSUBL	srRCL22_C	number of lives left
	?C=0	A	none?
	GOYES	maGAME_OVER	outta here! (game over)
	GOTO	maSTART_OVER	start that level again

maGAME_OVER	LCHEX	#000CC	about 1 second
	GOSUBL	waWAIT	wait a sec.t
	GOSUBL	srRCL33_C	get practice indicator
	LAHEX	#FFFFF
	?A#C	A	are we in practice mode?
	GOYES	maGO_DH	you don't get a high-score
	GOSUB	chCHECK_HIGH	is this a high score? (RTN)
maGO_DH	GOLONG	dhHIGH	dhHIGH does a GOTO mnMENU

maEND	GOSBVL	=GETPTR	restore regs
	LC(5)	=FALSE	to clear sysflag -56 (sound on)
	?ST=0	11	is sound ON?
	GOYES	maE_SWRITE
	LC(5)	=TRUE	to set sysflag -56 (sound off)
maE_SWRITE	DAT1=C	A	write out (TRUE/FALSE)

	INTON
	ST=1	15	allow interrupts again

	GOSBVL	=GETPTR	restore regs (D0, mostly)
	A=DAT0	A	back to RPL
	D0=D0+	5
	PC=(A)


* This routine will check if your score (in sr23) is a high.
* If not, it returns immediately.  If so, it will prompt
* you to type in your initials, then insert the high
* score into the proper place. (Bumping out score #5).
*
* For now, it will not prompt for initials, but only
* insert the score into the proper place.

chCHECK_HIGH	GOSUBL	srRCL32_C	get high-score data address
	LAHEX	#00034	offset to score #5
	C=C+A	A
	D0=C		set D0
	GOSUBL	srRCL23_C	get score.
	A=DAT0	A	read in score #5
	?C<=A	A	score too small?
	RTNYES		leave promptly

	LCHEX	#00004	(4+1) scores in list
	D=C	A
	GOSUBL	srRCL32_C	get high-score data @ again
	D0=C		@ of score #1
	GOSUBL	srRCL23_C	get score back
chNEXT	A=DAT0	A	read score
	?C>A	A	did you beat this one?
	GOYES	chWINNER
	D0=D0+	13	move to next score
	D=D-1	A	one less for counter
	GONC	chNEXT	check next

chWINNER	A=0	W	clear out A
	A=C	A	get new high score
	LCHEX	#00FFFFFF00000	a marker
	C=C+A	A	add on score
chW_NEXT	A=DAT0	13	get old score (and letters)
	DAT0=C	13	write new score here
	C=A	W	put old score here
	D0=D0+	13	move to next score
	D=D-1	A	dec counter
	GONC	chW_NEXT
	RTN

* -----------------------------------------------------
* The following routines display various graphics
* in a "window", then a delay, then they re-draw
* what used to be there.
* -----------------------------------------------------

grGET_READY	GOSUB	grTEMP
	R3=C.F	A	@ of temp area
	C=PC
	GOTO	grGR_NEXT

	NIBHEX	000000000000000000	line 0
	NIBHEX	CFFFFFFFFFFFFFFFF3
	NIBHEX	200000000000000004
	NIBHEX	200000100000000004
	NIBHEX	283F9F0C3E3E974444
	NIBHEX	24A2420A250959A244
	NIBHEX	228E020A1D1459A224
	NIBHEX	2A320208240C190124
	NIBHEX	221111042222980104
	NIBHEX	2C9F830E4F91D78014
	NIBHEX	201000004000008004
	NIBHEX	2C0000008100006004
	NIBHEX	200000000000000004
	NIBHEX	CFFFFFFFFFFFFFFFF3
	NIBHEX	000000000000000000	line 14

grGR_NEXT	C=C+CON	A,4
	R4=C.F	A	@ of graphic

	GOSUBL	srRCL9_A	screen @
	LCHEX	#00334	{#10h #18h}
	A=A+C	A
	D0=A

	LCHEX	#0000E	14+1 lines
	B=C	A
grGR_LINE	LCHEX	#00001	1+1 parts per line
	D=C	A

grGR_PART	C=R3.F	A	@ of temp area
	D1=C
	C=C+CON	A,9	increment C by 9
	R3=C.F	A	save it
	A=DAT0	9	get 9 screen nibbles
	DAT1=A	9	save them
	C=R4.F	A	@ of graphic
	D1=C
	C=C+CON	A,9	increment C by 9
	R4=C.F	A	save it
	A=DAT1	9	read 9 graphic nibbles
	DAT0=A	9	write 9 nibbles to screen
	D0=D0+	9	increment D0

	D=D-1	A
	GONC	grGR_PART	do next part

	AD0EX
	LCHEX	#00010	#22h - #12h
	A=A+C	A
	D0=A

	B=B-1	A	one less line
	GONC	grGR_LINE	do next line

	LCHEX	#03000	a couple of seconds
	B=C	A
grCHECK_KEY	LCHEX	#1FF	| wait here until
	OUT=C		| they stop pressing
	GOSBVL	=CINRTN	| a key
	LAHEX	#0003F	|
	A=A&C	A	|
	?A=0	A	|
	GOYES	grNO_KEY	|
	B=B-1	A	don't wait forever, though
	GONC	grCHECK_KEY

grNO_KEY	LCHEX	#00198	about 2 seconds
	GOSUBL	waWAIT	delay, pressing a key gets out

	LCHEX	#03000
	B=C	A
grCHECK_KEY2	LCHEX	#1FF	| wait here until
	OUT=C		| they stop pressing
	GOSBVL	=CINRTN	| a key
	LAHEX	#0003F	|
	A=A&C	A	|
	?A=0	A	|
	GOYES	grNO_KEY2	|
	B=B-1	A	don't wait forever, though
	GONC	grCHECK_KEY2

grNO_KEY2	GOSUB	grTEMP
	D1=C		temp area

	GOSUBL	srRCL9_A	screen @
	LCHEX	#00334	{#10h #18h}
	A=A+C	A
	D0=A

	LCHEX	#0000E	14+1 lines
	B=C	A
grGR_LINE2	LCHEX	#00001	1+1 parts per line
	D=C	A

grGR_PART2	A=DAT1	9	get 9 saved nibbles
	DAT0=A	9	put them back on the screen
	D1=D1+	9
	D0=D0+	9

	D=D-1	A
	GONC	grGR_PART2	do next part

	AD0EX
	LCHEX	#00010	#22h - #12h
	A=A+C	A
	D0=A

	B=B-1	A	one less line
	GONC	grGR_LINE2	do next line
	RTN


grTEMP	A=R0.F	A	@ of memory area
	LC(5)	VARS	offset past vars
	C=C+A	A	add the @ in C
	RTN		return value in C, field A


grBONUS	C=PC
	GOTO	grB_NEXT

	NIBHEX	000000000000	line 0
	NIBHEX	CFFFFFFFFFF3
	NIBHEX	200000000004
	NIBHEX	200000100004
	NIBHEX	2C36C8C4C024
	NIBHEX	2A29A9A42024
	NIBHEX	2AB8AA844014
	NIBHEX	28A88A428014
	NIBHEX	24A48C429004
	NIBHEX	2E1368856804
	NIBHEX	200008000004
	NIBHEX	200000300004
	NIBHEX	200000000004
	NIBHEX	CFFFFFFFFFF3
	NIBHEX	000000000000	line 14

grB_NEXT	C=C+CON	A,4
	D1=C	A	@ of graphic

	GOSUBL	srRCL9_A	screen @
	LCHEX	#00337	{#13h #18h}
	A=A+C	A
	D0=A

	LCHEX	#0000E	14+1 lines
	B=C	A
	LCHEX	#00022

grB_LINE	A=DAT1	12	read 12 graphic nibbles
	DAT0=A	12	write 12 nibbles to screen
	D1=D1+	12	increment D1

	AD0EX
	A=A+C	A
	D0=A		set new D0

	B=B-1	A	one less line
	GONC	grB_LINE	do next line
	RTN


grNO_BONUS	C=PC
	GOTO	grNB_NEXT

	NIBHEX	0000000000000000	line 0
	NIBHEX	CFFFFFFFFFFFFFF3
	NIBHEX	2000000000000004
	NIBHEX	2080000000400004
	NIBHEX	264C00F813231384
	NIBHEX	2D4218A4A6A29084
	NIBHEX	255118E2AA221144
	NIBHEX	245110A22A290244
	NIBHEX	2469009212394204
	NIBHEX	2346087C81269124
	NIBHEX	2040000000200004
	NIBHEX	2081000000C00004
	NIBHEX	2000000000000004
	NIBHEX	CFFFFFFFFFFFFFF3
	NIBHEX	0000000000000000	line 14

grNB_NEXT	C=C+CON	A,4
	D1=C

	GOSUBL	srRCL9_A	screen @
	LCHEX	#00335	{#11h #18h}
	A=A+C	A
	D0=A

	LCHEX	#0000E	14+1 lines
	B=C	A
	LCHEX	#00022

grNB_LINE	A=DAT1	16	read 16 graphic nibbles
	DAT0=A	16	write 16 nibbles to screen
	D1=D1+	16	increment D1

	AD0EX
	A=A+C	A
	D0=A

	B=B-1	A	one less line
	GONC	grNB_LINE	do next line
	RTN

grGAME_OVER	C=PC
	GOTO	grGO_NEXT

	NIBHEX	000000000000000000	line 0
	NIBHEX	CFFFFFFFFFFFFFFFF3
	NIBHEX	200000000000000004
	NIBHEX	200000000000800004
	NIBHEX	283FC16F106C8C7E14
	NIBHEX	24AC2992009A4A0514
	NIBHEX	220AA69E088A4A3D04
	NIBHEX	2A3E8A020888280414
	NIBHEX	221988011848144214
	NIBHEX	2CD8689F00380E3724
	NIBHEX	201000000000000024
	NIBHEX	2C00000000000000C4
	NIBHEX	200000000000000004
	NIBHEX	CFFFFFFFFFFFFFFFF3
	NIBHEX	000000000000000000	line 14

grGO_NEXT	C=C+CON	A,4
	D1=C		address of graphic

	GOSUBL	srRCL9_A	screen @
	LCHEX	#00334	{#10h #18h}
	A=A+C	A
	D0=A

	LCHEX	#0000E	14+1 lines
	B=C	A
grGO_LINE	LCHEX	#00001	1+1 parts per line
	D=C	A

grGO_PART	A=DAT1	9	read 9 graphic nibbles
	DAT0=A	9	write 9 nibbles to screen
	D0=D0+	9	increment D0
	D1=D1+	9

	D=D-1	A
	GONC	grGO_PART	do next part

	AD0EX
	LCHEX	#00010	#22h - #12h
	A=A+C	A
	D0=A

	B=B-1	A	one less line
	GONC	grGO_LINE	do next line

	RTN



* This will erase one line from the bonus area.
* If there are no lines, then nothing will happen.
* Note that, upon return, field A of A != 0 if there
* is bonus left.

beBONUS_ERS	GOSUBL	srRCL9_A	screen @
	LCHEX	#003F4	{#68h #1Dh}
	A=A+C	A
	D0=A
	A=0	W
	C=0	W
	A=DAT0	7	get 7 nibbles of the screen
	LCHEX	#3FFFFFF	a mask of what we want
	A=A&C	W	mask it off
	ASRB		lose a bit
	LCHEX	#4000000	get that border back
	A=A!C	W	put it on
	DAT0=A	7	write it on the screen
	LCHEX	#00022
	B=C	A
	CD0EX
	C=C+B	A	down one line
	D0=C		into D0
	DAT0=A	7	draw the next line too
	RTN		that's all it takes!

* This will fill the bonus area

beBONUS_DRAW	GOSUBL	srRCL9_A	screen @
	LCHEX	#003F4	{#68h #1Dh}
	A=A+C	A
	D0=A
	LAHEX	#7FFFFFF	a full bonus, and a border
	DAT0=A	7	draw it
	LCHEX	#00022
	B=C	A
	CD0EX
	C=C+B	A	down one line
	D0=C
	DAT0=A	7	draw this one too
	RTN		all done


dlDRAW_LIVES	GOSUBL	srRCL9_A	screen @
	LCHEX	#007B1	{#7Ch #39h}
	C=C+A	A	|
	D0=C		| erase 2 nibbles
	C=0	A	| in case there was
	DAT0=C	2	| a number there

	LCHEX	#007CE	{#68h #3Ah}
	C=C+A	A
	D0=C		set D0

	C=PC
	GOTO	dlLIVES_LEFT

	NIBHEX	E0783C1	|This is the data for the graphic
	NIBHEX	F9FC7E3	|that shows how many lives left.
	NIBHEX	F9FC7E3	|This algorithm will also draw
	NIBHEX	F9FC7E3	|white if there is only 1 life
	NIBHEX	E0783C1	|left, so you don't have to erase.

dlLIVES_LEFT	C=C+CON	A,4	C is now @ of data
	D1=C		put that into D1

	LCHEX	#00004	(4+1) times
dlDO_LINE	R4=C.F	A
	GOSUB	srRCL22_C	# of lives left
	B=C	A
	B=B-1	A	display 1 less than you have (one is playing)
	LCHEX	#00004	never display more than 4 lives
	?B<=C	A	do you have 4 or less?
	GOYES	dlA_START	display what you have
	B=C	A	make it only 4
	B=B-1	A	now make it 3 to give room for a #
dlA_START	A=0	W
dlANOTHER	?B=0	A
	GOYES	dlNO_ADD
	LCHEX	#00006	(6+1) times
	D=C	A
dlSHIFT	A=A+A	W	shift bits left
	A=A+1	A	make bit 0 a 1
	D=D-1	A
	GONC	dlSHIFT
	B=B-1	A
	GOTO	dlANOTHER

dlNO_ADD	C=DAT1	7	read one line of data
	C=C&A	W	mask out what we want
	DAT0=C	7	write to the screen
	D1=D1+	7	increment to next line
	LCHEX	#00022
	AD0EX
	A=A+C	A
	D0=A
	C=R4.F	A
	C=C-1	A
	GONC	dlDO_LINE	do some more

	GOSUB	srRCL22_C	number of lives again
	B=C	A
	B=B-1	A	always display 1 less
	LCHEX	#00005	too many...
	?B<C	A	do you have 4 or less?
	RTNYES		then go home

	LCHEX	#0000A	way too big
	?B>=C	A	do you have 10 or more?
	GOYES	dlPLUS

	C=PC
	GOTO	dlNEXT

	NIBHEX	530063007300
	NIBHEX	83009300

dlNEXT	C=C+CON	A,4
	D0=C		point D0 to "5"
	B=B-CON	A,5	if you have 5, it's a 0
dlCH_ADD	?B=0	A	add some?
	GOYES	dlNOPE
	D0=D0+	4
	B=B-1	A
	GOTO	dlCH_ADD

dlNOPE	LCHEX	#01DA8	{#7Dh #39h}
	GOLONG	fdFONT	display the num (AUTO RTN)

dlPLUS	GOSUB	srRCL9_A	PICT @
	LCHEX	#007D3	{#7Ch #3Ah}
	C=C+A	A
	D0=C

	LCHEX	#00022
	B=C	A
	LAHEX	#08	first 2 nibbles
	DAT0=A	2	write to screen
	CD0EX		|
	C=C+B	A	| down 1 line
	D0=C		|
	DAT0=A	2	write to screen
	CD0EX		|
	C=C+B	A	| down 1 line
	D0=C		|
	LAHEX	#3E
	DAT0=A	2	write to screen
	CD0EX		|
	C=C+B	A	| down 1 line
	D0=C		|
	LAHEX	#08
	DAT0=A	2	write to screen
	CD0EX		|
	C=C+B	A	| down 1 line
	D0=C		|
	DAT0=A	2	write to screen

	RTN		end of dlDRAW_LIVES


meMAIN_EVENT	ST=0	1	key delay
	ST=0	2	key delay check
	ST=0	3	key move direction (0==NORMAL)
	ST=0	4	you're not dead (yet!)
	ST=0	5	you haven't won (yet!)
	ST=0	10	set to draw mode (for db)

	LCHEX	#080
	OUT=C
	GOSBVL	=CINRTN
	LAHEX	#0001F
	A=A&C	A
	LCHEX	#00004
	?A#C	A
	GOYES	meMAIN_LSTRT
	LCHEX	#040
	OUT=C
	GOSBVL	=CINRTN
	LAHEX	#0001F
	A=A&C	A
	LCHEX	#00008
	?A#C	A
	GOYES	meMAIN_LSTRT
	LCHEX	#100
	OUT=C
	GOSBVL	=CINRTN
	LAHEX	#0001F
	A=A&C	A
	LCHEX	#00002
	?A#C	A
	GOYES	meMAIN_LSTRT
	GOSUBL	srSTO43_C	start spreading myths
	GOSUBL	soMYTH	make a mythical sound

* So what is the big myth anyway?  As is written in plain assembly above,
* whenever the GET READY banner is displayed, if you are holding down
* the keys [J] [O] [E] simultaneously when the banner disappears, then
* rumor has it that you will have unlimited lives.  Once you are in the
* "mythical" mode, you can press [U] or [D] whenever the GET READY banner
* is displayed.  When the banner goes away, you will automatically go
* up or down one level respectively.  Great debugging tool, and even greater
* cheat tool.  Joe is the name of my son.  At the time I wrote this, I had
* only two children, and Kathryn was just too many keys to make you hold
* down at one time! :)  Since this time, Jessica and Emilyn have been born,
* but I still think that Joe was the wise choice.

meMAIN_LSTRT	D1=(5)	=TIMER2
	C=0	A
	C=DAT1	4	get the value of TIMER2
	GOSUBL	srSTO47_C	save (me) starting time

meMAIN_LOOP	LCHEX	#010
	OUT=C
	GOSBVL	=CINRTN
	?CBIT=0	0	[<=] key
	GOYES	meNO_BACK
	C=0	A
	GOSUBL	srSTO43_C	stop spreading myths
	LCHEX	#1	since C is already #00000
	GOSUB	srSTO22_C	set lives to 1
	GOSUBL	hbDEAD	make sound
	GOLONG	maMAIN_RTN	instead of RTN

meNO_BACK	LCHEX	#010
	OUT=C
	GOSBVL	=CINRTN
	?CBIT=0	1	[DEL] key
	GOYES	meNO_DEL
	ST=1	4	set YOU'RE DEAD flag
	GOSUBL	hbDEAD
	GOLONG	maMAIN_RTN	instead of RTN

meNO_DEL	LCHEX	#010
	OUT=C
	GOSBVL	=CINRTN
	?CBIT=0	4	[ENTER] key (pause)
	GOYES	meNO_ENT

meSTILL_ENT	LCHEX	#010	| wait here until
	OUT=C		| they stop pressing
	GOSBVL	=CINRTN	| the [ENTER] key
	?CBIT=1	4	|
	GOYES	meSTILL_ENT	|

	GOSUB	pkPAUSE_KEY

meNO_ENT	?ST=0	1	no bounce recently?
	GOYES	meKEY_LEFT	check keys
	?ST=1	2	Check keys anyway?
	GOYES	meSET1
	ST=1	2
	GOTO	meCONT

meSET1	ST=0	1
meKEY_LEFT	ST=0	2	make sure 2 is zero
	LCHEX	#006
	OUT=C
	GOSBVL	=CINRTN
	?CBIT=0	4	[A] or [G] keys
	GOYES	meKEY_RIGHT

	GOSUB	srRCL2_A
	LCHEX	#00003
	A=A&C	A
	?ST=0	3
	GOYES	meKL_NORMAL
	LCHEX	#00008	opposite dir (right)
	GOTO	meKL_CONT
meKL_NORMAL	LCHEX	#00004	left dir
meKL_CONT	A=A!C	A
	GOSUBL	srSTO2_A
	GOTO	meCONT

meKEY_RIGHT	LCHEX	#180
	OUT=C
	GOSBVL	=CINRTN
	?CBIT=0	0	[F] or [L] keys
	GOYES	meNONE

	GOSUB	srRCL2_A
	LCHEX	#00003
	A=A&C	A
	?ST=0	3
	GOYES	meKR_NORMAL
	LCHEX	#00004
	GOTO	meKR_CONT
meKR_NORMAL	LCHEX	#00008
meKR_CONT	A=A!C	A
	GOSUBL	srSTO2_A
	GOTO	meCONT

meNONE	GOSUB	srRCL2_A
	LCHEX	#00003
	A=A&C	A
	GOSUBL	srSTO2_A

meCONT	GOSUBL	mvMOVE
	?ST=0	4	did you not die?
	GOYES	meC_NEXT1
	GOLONG	maMAIN_RTN	instead of RTN
meC_NEXT1	?ST=0	5	did you not win?
	GOYES	meC_NEXT2
	GOLONG	maMAIN_RTN	instead of RTN

meC_NEXT2	GOSUB	srRCL25_C	bonus time
	C=C-1	A
	GOSUB	srSTO25_C
	?C#0	A	still more?
	GOYES	meDELAY	yes? skip
	LC(5)	BONUS_TIME	reset bonus time
	GOSUB	srSTO25_C
	GOSUB	beBONUS_ERS	erase a bonus tick

meDELAY	GOSUB	srRCL18_C	minumum time for main event loop (ticks)
	R2=C.F	A	store n here
	GOSUBL	srRCL47_C	get starting time
	GOSUBL	udDELAYcont	delay the remaining time.
	GOTO	meMAIN_LOOP


pkPAUSE_KEY	LCHEX	#5FF	all keys (even [ON])
	OUT=C
	GOSBVL	=CINRTN
	LAHEX	#0803F	all keys (even [ON])
	A=A&C	A
	?A=0	A	no press?
	GOYES	pkPAUSE_KEY

	LAHEX	#08000	[ON]
	A=A&C	A
	?A=0	A	not this one?
	GOYES	pkWAIT
	GOLONG	maEND	return immediately to RPL.

pkWAIT	LCHEX	#1FF
	OUT=C
	GOSBVL	=CINRTN
	LAHEX	#0003F	all keys (not [ON])
	A=A&C	A
	?A#0	A	you're still pressing a key?
	GOYES	pkWAIT	wait here until you stop.
	RTN		done with PAUSE_KEY


* This will display either the BONUS or NO BONUS
* banners, and then add the bonus to the score,
* if necessary.

cnCONGRATS	GOSUB	cnMUSIC	play a little tune
	LCHEX	#00066	about 1/2 second
	GOSUBL	waWAIT

	GOSUB	srRCL9_A	screen @
	LCHEX	#003F4	{#1Ah #1Dh}
	A=A+C	A
	D0=A
	A=DAT0	1	get a nibble
	?ABIT=0	0	if bit 0 == 0, no bonus
	GOYES	cnNO_BONUS

	GOSUBL	grBONUS	display the BONUS banner

cnBONUS	GOSUB	beBONUS_ERS	erase one line
	?A=0	A	no more bonus?
	GOYES	cnALMOST_END
	LCHEX	#00025
	GOSUBL	scADD_SCORE
	GOSUBL	soBONUS
	LCHEX	#000C0	ticks
	GOSUBL	udDELAY
	GOTO	cnBONUS

cnALMOST_END	LCHEX	#00025
	GOSUBL	scADD_SCORE	get that last one!
	GOSUBL	soBONUS
	GOTO	cn_END

cnNO_BONUS	GOSUBL	grNO_BONUS	display the NO BONUS banner

cn_END	LCHEX	#00132	about 1.5 seconds
	GOSUBL	waWAIT	delay, pressing a key gets out
	RTN

cnMUSIC	?ST=1	11	sound off?
	RTNYES		outta here!

	LCHEX	#00066	about 1/2 second
	GOSUBL	waWAIT

	LCHEX	#00080	starting pitch
	D=C	A

cn_START	LCHEX	#00018	length
	B=C	A

cn_AGAIN	LCHEX	#800
	OUT=C
	C=D	A	current pitch
cn_DEC1	C=C-1	A
	GONC	cn_DEC1
	C=0	A
	OUT=C
	C=D	A	current pitch
cn_DEC2	C=C-1	A
	GONC	cn_DEC2
	B=B-1	A
	GONC	cn_AGAIN

	LCHEX	#00010
	D=D-C	A
	?D#0	A
	GOYES	cn_START
	RTN

* --------------------------------------
*
* This is a variable storage/retrieval system.
*
* Before any of these routines can be used, you
* must have RAM space pointed to by R0.
* If you call srRCL_C, or srSTO_C, then D0 and A field A
* remain unchanged.  Likewise, if you call srRCL_A or
* srSTO_A, then D0 and C field A remain unchanged even
* though they are used.  Also the data stored with
* srSTO_C and sr_STO_A is still in field A of the reg
* on exit.
*
* --------------------------------------


srRCL3_C	LCHEX	#00003	|  These simple routines
	GOTO	srRCL_C	|  make it possible to recall
srRCL4_C	LCHEX	#00004	|  a specific variable number
	GOTO	srRCL_C	|  with one single GOSUB.  More
srRCL5_C	LCHEX	#00005	|  can be added, if necessary.
	GOTO	srRCL_C	|
srRCL6_C	LCHEX	#00006	|
	GOTO	srRCL_C	|
srRCL7_C	LCHEX	#00007	|
	GOTO	srRCL_C	|
srRCL8_C	LCHEX	#00008	|
	GOTO	srRCL_C	|
srRCL9_C	LCHEX	#00009	|
	GOTO	srRCL_C	|
srRCL15_C	LCHEX	#0000F	|
	GOTO	srRCL_C	|
srRCL18_C	LCHEX	#00012	|
	GOTO	srRCL_C	|
srRCL19_C	LCHEX	#00013	|
	GOTO	srRCL_C	|
srRCL20_C	LCHEX	#00014	|
	GOTO	srRCL_C	|
srRCL22_C	LCHEX	#00016	|
	GOTO	srRCL_C	|
srRCL23_C	LCHEX	#00017	|
	GOTO	srRCL_C	|
srRCL24_C	LCHEX	#00018	|
	GOTO	srRCL_C	|
srRCL25_C	LCHEX	#00019	|
	GOTO	srRCL_C	|
srRCL26_C	LCHEX	#0001A	|
	GOTO	srRCL_C	|
srRCL27_C	LCHEX	#0001B	|
	GOTO	srRCL_C	|
srRCL28_C	LCHEX	#0001C	|
	GOTO	srRCL_C	|
srRCL29_C	LCHEX	#0001D	|
	GOTO	srRCL_C	|
srRCL30_C	LCHEX	#0001E	|
	GOTO	srRCL_C	|
srRCL31_C	LCHEX	#0001F	|
	GOTO	srRCL_C	|
srRCL32_C	LCHEX	#00020	|
	GOTO	srRCL_C	|
srRCL33_C	LCHEX	#00021	|
	GOTO	srRCL_C	|
srRCL34_C	LCHEX	#00022	|
	GOTO	srRCL_C	|
srRCL35_C	LCHEX	#00023	|
	GOTO	srRCL_C	|
srRCL36_C	LCHEX	#00024	|
	GOTO	srRCL_C	|
srRCL37_C	LCHEX	#00025	|
	GOTO	srRCL_C	|
srRCL38_C	LCHEX	#00026	|
	GOTO	srRCL_C	|
srRCL39_C	LCHEX	#00027	|
	GOTO	srRCL_C	|
srRCL40_C	LCHEX	#00028	|
	GOTO	srRCL_C	|
srRCL41_C	LCHEX	#00029	|
	GOTO	srRCL_C	|
srRCL42_C	LCHEX	#0002A	|
	GOTO	srRCL_C	|
srRCL43_C	LCHEX	#0002B	|
	GOTO	srRCL_C	|
srRCL44_C	LCHEX	#0002C	|
	GOTO	srRCL_C	|
srRCL45_C	LCHEX	#0002D	|
	GOTO	srRCL_C	|
srRCL46_C	LCHEX	#0002E	|
	GOTO	srRCL_C	|
srRCL47_C	LCHEX	#0002F	|


srRCL_C	ACEX	A
	CD0EX
	RSTK=C
	C=A	A
	C=C+C	A
	C=C+C	A
	C=C+A	A
	A=R0.F	A
	A=A+C	A
	AD0EX
	C=DAT0	A
	CD0EX
	C=RSTK
	CD0EX
	RTN


srRCL0_A	A=0	A	|  These simple routines
	GOTO	srRCL_A	|  make it possible to recall
srRCL1_A	LAHEX	#00001	|  a specific variable number
	GOTO	srRCL_A	|  with one single GOSUB.  More
srRCL2_A	LAHEX	#00002	|  can be added, if necessary.
	GOTO	srRCL_A	|
srRCL3_A	LAHEX	#00003	|
	GOTO	srRCL_A	|
srRCL4_A	LAHEX	#00004	|
	GOTO	srRCL_A	|
srRCL6_A	LAHEX	#00006	|
	GOTO	srRCL_A	|
srRCL7_A	LAHEX	#00007	|
	GOTO	srRCL_A	|
srRCL8_A	LAHEX	#00008	|
	GOTO	srRCL_A	|
srRCL9_A	LAHEX	#00009	|
	GOTO	srRCL_A	|
srRCL15_A	LAHEX	#0000F	|
	GOTO	srRCL_A	|
srRCL16_A	LAHEX	#00010	|
	GOTO	srRCL_A	|
srRCL17_A	LAHEX	#00011	|
	GOTO	srRCL_A	|
srRCL19_A	LAHEX	#00013	|
	GOTO	srRCL_A	|
srRCL21_A	LAHEX	#00015	|
	GOTO	srRCL_A	|
srRCL23_A	LAHEX	#00017	|
	GOTO	srRCL_A	|
srRCL24_A	LAHEX	#00018	|
	GOTO	srRCL_A	|
srRCL27_A	LAHEX	#0001B	|
	GOTO	srRCL_A	|
srRCL28_A	LAHEX	#0001C	|

srRCL_A	CD0EX
	RSTK=C
	C=A	A
	A=A+A	A
	A=A+A	A
	A=A+C	A
	C=R0.F	A
	C=C+A	A
	CD0EX
	A=DAT0	A
	CD0EX
	C=RSTK
	CD0EX
	RTN


srSTO0_C	GOSBVL	=CSLW5	|  These simple routines
	C=0	A	|  make it possible to store
	GOTO	srSTO_C	|  a specific variable number
srSTO1_C	GOSBVL	=CSLW5	|  with one single GOSUB.  More
	LCHEX	#00001	|  can be added, if necessary.
	GOTO	srSTO_C	|
srSTO2_C	GOSBVL	=CSLW5	|  Also, the data being stored
	LCHEX	#00002	|  (in C, field A) is preserved,
	GOTO	srSTO_C	|  and on exit, it is still in
srSTO3_C	GOSBVL	=CSLW5	|  C, field A.	You may call
	LCHEX	#00003	|  srSTO_C directly, if you first
	GOTO	srSTO_C	|  call =CSLW5 to shift the data over,
srSTO4_C	GOSBVL	=CSLW5	|  then place the Var# in C, field A
	LCHEX	#00004	|  On exit, the var data is in C, field A.
	GOTO	srSTO_C	|
srSTO5_C	GOSBVL	=CSLW5	|
	LCHEX	#00005	|
	GOTO	srSTO_C	|
srSTO7_C	GOSBVL	=CSLW5	|
	LCHEX	#00007	|
	GOTO	srSTO_C	|
srSTO8_C	GOSBVL	=CSLW5	|
	LCHEX	#00008	|
	GOTO	srSTO_C	|
srSTO9_C	GOSBVL	=CSLW5	|
	LCHEX	#00009	|
	GOTO	srSTO_C	|
srSTO15_C	GOSBVL	=CSLW5	|
	LCHEX	#0000F	|
	GOTO	srSTO_C	|
srSTO16_C	GOSBVL	=CSLW5	|
	LCHEX	#00010	|
	GOTO	srSTO_C	|
srSTO17_C	GOSBVL	=CSLW5	|
	LCHEX	#00011	|
	GOTO	srSTO_C	|
srSTO18_C	GOSBVL	=CSLW5	|
	LCHEX	#00012	|
	GOTO	srSTO_C	|
srSTO19_C	GOSBVL	=CSLW5	|
	LCHEX	#00013	|
	GOTO	srSTO_C	|
srSTO20_C	GOSBVL	=CSLW5	|
	LCHEX	#00014	|
	GOTO	srSTO_C	|
srSTO21_C	GOSBVL	=CSLW5	|
	LCHEX	#00015	|
	GOTO	srSTO_C	|
srSTO22_C	GOSBVL	=CSLW5	|
	LCHEX	#00016	|
	GOTO	srSTO_C	|
srSTO23_C	GOSBVL	=CSLW5	|
	LCHEX	#00017	|
	GOTO	srSTO_C	|
srSTO24_C	GOSBVL	=CSLW5	|
	LCHEX	#00018	|
	GOTO	srSTO_C	|
srSTO25_C	GOSBVL	=CSLW5	|
	LCHEX	#00019	|
	GOTO	srSTO_C	|
srSTO26_C	GOSBVL	=CSLW5	|
	LCHEX	#0001A	|
	GOTO	srSTO_C	|
srSTO27_C	GOSBVL	=CSLW5	|
	LCHEX	#0001B	|
	GOTO	srSTO_C	|
srSTO28_C	GOSBVL	=CSLW5	|
	LCHEX	#0001C	|
	GOTO	srSTO_C	|
srSTO29_C	GOSBVL	=CSLW5	|
	LCHEX	#0001D	|
	GOTO	srSTO_C	|
srSTO30_C	GOSBVL	=CSLW5	|
	LCHEX	#0001E	|
	GOTO	srSTO_C	|
srSTO31_C	GOSBVL	=CSLW5	|
	LCHEX	#0001F	|
	GOTO	srSTO_C	|
srSTO32_C	GOSBVL	=CSLW5	|
	LCHEX	#00020	|
	GOTO	srSTO_C	|
srSTO33_C	GOSBVL	=CSLW5	|
	LCHEX	#00021	|
	GOTO	srSTO_C	|
srSTO34_C	GOSBVL	=CSLW5	|
	LCHEX	#00022	|
	GOTO	srSTO_C	|
srSTO35_C	GOSBVL	=CSLW5	|
	LCHEX	#00023	|
	GOTO	srSTO_C	|
srSTO36_C	GOSBVL	=CSLW5	|
	LCHEX	#00024	|
	GOTO	srSTO_C	|
srSTO37_C	GOSBVL	=CSLW5	|
	LCHEX	#00025	|
	GOTO	srSTO_C	|
srSTO38_C	GOSBVL	=CSLW5	|
	LCHEX	#00026	|
	GOTO	srSTO_C	|
srSTO39_C	GOSBVL	=CSLW5	|
	LCHEX	#00027	|
	GOTO	srSTO_C	|
srSTO40_C	GOSBVL	=CSLW5	|
	LCHEX	#00028	|
	GOTO	srSTO_C	|
srSTO41_C	GOSBVL	=CSLW5	|
	LCHEX	#00029	|
	GOTO	srSTO_C	|
srSTO42_C	GOSBVL	=CSLW5	|
	LCHEX	#0002A	|
	GOTO	srSTO_C	|
srSTO43_C	GOSBVL	=CSLW5	|
	LCHEX	#0002B	|
	GOTO	srSTO_C	|
srSTO44_C	GOSBVL	=CSLW5	|
	LCHEX	#0002C	|
	GOTO	srSTO_C	|
srSTO45_C	GOSBVL	=CSLW5	|
	LCHEX	#0002D	|
	GOTO	srSTO_C	|
srSTO46_C	GOSBVL	=CSLW5	|
	LCHEX	#0002E	|
	GOTO	srSTO_C	|
srSTO47_C	GOSBVL	=CSLW5	|
	LCHEX	#0002F	|

srSTO_C	GOSBVL	=ASLW5	preserve A, field A
	AD0EX		put D0 into A, field A
	GOSBVL	=ASLW5	preserve it (D0)
	A=C	A	|
	C=C+C	A	| Multiply Var# by 5
	C=C+C	A	|
	C=C+A	A	|
	A=R0.F	A	get RAM address of Vars
	A=A+C	A	add the offset (Var# * 5)
	D0=A		put it in D0
	GOSBVL	=CSRW5	shift C to get var data
	DAT0=C	A	write var to RAM
	GOSBVL	=ASRW5	restore D0
	D0=A		put it in D0
	GOSBVL	=ASRW5	restore A, field A
	RTN

srSTO2_A	GOSBVL	=ASLW5	|  These simple routines
	LAHEX	#00002	|  make it possible to store
	GOTO	srSTO_A	|  a specific variable number
srSTO4_A	GOSBVL	=ASLW5	|  with one single GOSUB.  More
	LAHEX	#00004	|  can be added, if necessary.
	GOTO	srSTO_A	|
srSTO6_A	GOSBVL	=ASLW5	|
	LAHEX	#00006	|
	GOTO	srSTO_A	|
srSTO16_A	GOSBVL	=ASLW5	|
	LAHEX	#00010	|
	GOTO	srSTO_A	|
srSTO17_A	GOSBVL	=ASLW5	|
	LAHEX	#00011	|
	GOTO	srSTO_A	|
srSTO28_A	GOSBVL	=ASLW5	|
	LAHEX	#0001C	|


srSTO_A	GOSBVL	=CSLW5	preserve C, field A
	CD0EX		put D0 into C, field A
	GOSBVL	=CSLW5	preserve it (D0)
	C=A	A	|
	A=A+A	A	| Multiply Var# by 5
	A=A+A	A	|
	A=A+C	A	|
	C=R0.F	A	get RAM address of Vars
	C=C+A	A	add the offset (Var # 5)
	D0=C		put it in D0
	GOSBVL	=ASRW5	shift A to get var data
	DAT0=A	A	write var to RAM
	GOSBVL	=CSRW5	restore D0
	D0=C		put it in D0
	GOSBVL	=CSRW5	restore C, field A
	RTN


* -------------------------------------
* MOVE needs 3 arguments
*
* Var #0 == Current x coord
* Var #1 == Current y coord
* Var #2 == direction to move (0 for no move, just draw)
*
* MOVE will change x & y in the direction of R2.


mvMOVE	GOSUB	srRCL0_A	get X coord
	R1=A.F	A	put it in temp X
	GOSUB	srRCL1_A	get Y coord
	R2=A.F	A	put it in temp y

	GOSUB	srRCL2_A	Direction
	R3=A.F	A	Temp Dir used in cb
	?ABIT=0	0	Going up?
	GOYES	mvCHDN	no? check down

mvUP	C=R2.F	A	temp Y
	C=C-1	A	up one
	GOC	mvCHDN	less than 0? no good!
	R2=C.F	A	store it (in te=MPY)

mvCHDN	?ABIT=0	1	going down?
	GOYES	mvCHLEFT	no? check left

mvDOWN	LCHEX	#0003C	stopping point (#40h - #5h + #1h)
	B=C	A	put it in B
	C=R2.F	A	temp Y
	C=C+1	A	down 1
	?B<=C	A	down too far?
	GOYES	mvCHLEFT	yes? check left
	R2=C.F	A	store it (in te=MPY)

mvCHLEFT	?ABIT=0	2	going left?
	GOYES	mvCHRGT	no? check right

mvLEFT	C=R1.F	A	temp X
	C=C-1	A	left one
	GOC	mvCHRGT	too far left
	R1=C.F	A	store it (in tempX)

mvCHRGT	?ABIT=0	3	going right?
	GOYES	dbDRAW_BALL	no? draw it.
mvRIGHT	LCHEX	#0007F	limit (#83h - #5h + #1h) (#83h == 131)
	B=C	A
	C=R1.F	A	temp X
	C=C+1	A
	?B<=C	A	too far?
	GOYES	dbDRAW_BALL	draw it
	R1=C.F	A	store it (in tempX)

dbDRAW_BALL	C=PC
	GOTO	dbSTART

	NIBHEX	E0F1F1F1E0		dbSHAPE

	NIBHEX	000000000110E00	dbUPMASK
	NIBHEX	000020020030C10	dbUPLEFTMASK
	NIBHEX	000800800810070	dbUPRIGHTMASK
	NIBHEX	E00110000000000	dbDOWNMASK
	NIBHEX	C10030020020000	dbDOWNLEFTMASK
	NIBHEX	070810800800000	dbDOWNRIGHTMASK
	NIBHEX	010020020020010	dbLEFTMASK
	NIBHEX	010800800800010	dbRIGHTMASK

dbSTART	C=C+CON	A,4	C is now set to @shape
	GOSUB	srSTO7_C	store it in Var#7

	LCHEX	#00022
	D=C	A	set D to Y incr.
	GOSUBL	srRCL9_C	PICT bitmap address (not GROB)
	B=0	A
	A=R1.F	A	tempX coord.
	SB=0
	ASRB.F	A	divide by 2
	?SB=0		was there a bit lost?
	GOYES	dbAGAIN1	no? continue
	B=B+1	A	have to shift it at least once
dbAGAIN1	SB=0
	ASRB.F	A	divide by 2
	?SB=0		was there a bit lost?
	GOYES	dbAGAIN2	no... continue
	B=B+1	A	need to shift once more...
	B=B+1	A	...and once more
dbAGAIN2	C=C+A	A	add X coord
	A=R2.F	A	te=MPY coord.

dbADDY	?A=0	A	add some?
	GOYES	dbNONE	no? done
	C=C+D	A	down 1 line
	A=A-1	A
	GOTO	dbADDY
dbNONE	GOSUB	srSTO3_C	store it.
	C=B	A	get shifts
	GOSUB	srSTO4_C	save it for later
	?ST=1	10	check if we should erase the ball
	GOYES	dbSKIP

	GOSUB	cbCHK_BOUNCE	see if we bounce
	?ST=1	4	did you die?
	RTNYES
	?ST=1	5	did you win?
	RTNYES
dbSKIP	GOSUBL	srRCL3_C	good thing we saved it.
	D0=C		put screen @ in D0
	GOSUBL	srRCL7_C	@ of ball
	D1=C		put it in D1
	GOSUBL	srRCL4_C	# of shifts
	B=C	A	put it here
	C=R1.F	A	get temp X
	GOSUB	srSTO0_C	put in X
	C=R2.F	A	get temp Y
	GOSUB	srSTO1_C	put in Y

	LCHEX	#00004	set loop to 5 times
	D=C	A
dbDO_NEXT	A=0	A
	A=DAT1	2	get one line of ball
	C=B	A	number of shifts
dbCHECK	?C=0	A	do some?
	GOYES	dbCONT	no? continue
	A=A+A	A	shift graphic
	C=C-1	A
	GOTO	dbCHECK

dbCONT	C=DAT0	2	get current screen
	?ST=0	10	draw or erase?
	GOYES	dbORIT	go draw
	A=-A-1	A	NOT(A)
	A=A&C	A	mask out what we want
	GOTO	dbWRITEIT	go write it
dbORIT	A=A!C	A	OR with screen
dbWRITEIT	DAT0=A	2	draw graphic

	LCHEX	#00022
	AD0EX
	A=A+C	A	down 1 line
	D0=A		set D0
	D1=D1+	2	set D1 next ball line

	D=D-1	A	decrement counter

	GONC	dbDO_NEXT


dbCHECKDIR	GOSUB	srRCL2_A	direction
	?ABIT=0	0	shape not going up?
	GOYES	dbCHDOWN	yes it isn't? -> go next
	?ABIT=1	2	shape going left?
	GOYES	dbUPLEFT	yes? do up-left
	?ABIT=1	3	shape going right?
	GOYES	dbUPRIGHT	yes? do up-right

dbUP	GOSUB	srRCL3_A	get screen pos
	A=A+C	A	down 1 line
	D0=A		set D0
	GOTO	erERASE

dbUPLEFT	GOSUB	srRCL3_A	get screen pos
	A=A+C	A	down 1 line
	D0=A		set D0
	D1=D1+	15	go to next erase mask
	GOTO	erERASE

dbUPRIGHT	GOSUB	srRCL3_A	get screen pos
	A=A+C	A	up one line
	A=A-1	A	back one nibble
	D0=A		set D0
	D1=D1+	15	|
	D1=D1+	15	|set to up-right mask
	GOTO	erERASE

dbCHDOWN	D1=D1+	15	|
	D1=D1+	15	| Point to down erase mask
	D1=D1+	15	|

	?ABIT=0	1	shape not going down?
	RTNYES		yes it isn't? -> not moving.
	?ABIT=1	2	going left?
	GOYES	dbDNLEFT	yes? do down-left
	?ABIT=1	3	going right?
	GOYES	dbDNRIGHT	yes? do down-right

dbDOWN	GOSUB	srRCL3_A	screen pos
	A=A-C	A	up one line
	D0=A		set D0
	GOTO	erERASE

dbDNLEFT	GOSUB	srRCL3_A	screen pos
	A=A-C	A	up one line
	D0=A		set D0
	D1=D1+	15	set to down-left erase mask
	GOTO	erERASE

dbDNRIGHT	GOSUB	srRCL3_A	screen pos
	A=A-C	A	up one line
	A=A-1	A	back one nibble
	D0=A		set D0
	D1=D1+	15	|
	D1=D1+	15	|set to down-right erase mask

* No need to do a GOTO ERASE...it comes next.


erERASE	GOSUBL	srRCL4_C	DEBUG
	B=C	A	DEBUG
	LCHEX	#00004	loop 5 times
	D=C	A
erDO_NEXT	A=0	A
	A=DAT1	3	get 1 line of mask
	C=B	A	number of shifts
erCHECK	?C=0	A	do some?
	GOYES	erCONT	no? continue
	A=A+A	A	shift graphic
	C=C-1	A
	GOTO	erCHECK

erCONT	C=DAT0	3	get current screen
	A=-A-1	A	A=NOT(A) (inverse)
	A=A&C	A	mask out what we want
	DAT0=A	3	put on screen

	LCHEX	#00022
	AD0EX		get D0
	A=A+C	A	down 1 line
	D0=A		set D0
	D1=D1+	3	set D1 to next mask line
	D=D-1	A	decrement counter

	GONC	erDO_NEXT

	RTN


cbCHK_BOUNCE	GOSUBL	srRCL3_A	screen pos
	D0=A		set D0
	A=R3.F	A	temp direction

	C=PC
	GOTO	cbSTART

	NIBHEX	000000000110E00	cbDNMASK
	NIBHEX	000010010810E00	cbDNRIGHTMASK
	NIBHEX	000100100300E00	cbDNLEFTMASK
	NIBHEX	E00110000000000	cbUPMASK
	NIBHEX	E00810010010000	cbUPRIGHTMASK
	NIBHEX	E00300100100000	cbUPLEFTMASK

cbSTART	C=C+CON	A,4	C is now @ cbDOWN mask
	D1=C		put it in D1
	?ABIT=1	0	ball going up?
	GOYES	cbCHUP	yes -> do up
	?ABIT=1	3	going right?
	GOYES	cbDNRIGHT	yes? do down-right
	?ABIT=0	2	shape not going left?
	GOYES	cbMASK	yes it isn't? go to down

cbDNLEFT	D1=D1+	15	|point to proper erase mask
cbDNRIGHT	D1=D1+	15	|depending on where you jumped to
	GOTO	cbMASK	|

cbCHUP	D1=D1+	15	|
	D1=D1+	15	|point to down erase mask
	D1=D1+	15	|
	?ABIT=0	0	ball not going up?
	RTNYES		no? -> not moving.
	?ABIT=1	3	going right?
	GOYES	cbUPRIGHT	yes? do up-right
	?ABIT=0	2	ball not going left?
	GOYES	cbMASK	yes it isn't? -> do up

cbUPLEFT	D1=D1+	15	|point to proper erase mask
cbUPRIGHT	D1=D1+	15	|depending on where you jumped to

* no GOTO, cbMASK must come next...

* ------------------------------
* | At this point, D1 should point to the proper
* | erase mask. (opposite of direction... DIR = UP
* |  -> MASK = DOWN, DIR = UP-LEFT -> MASK = DN-RIGHT.
* | D0 should point to the screen address that the
* | ball will be drawn to if no bounce.
* ------------------------------


cbMASK	LCHEX	#00004	set loop to 5 times
	D=C	A	use D
cbNEXT_LINE	A=0	A	clear
	A=DAT1	3	get 1 line of mask
	C=B	A	# of shifts
cbSHIFT	?C=0	A	do some?
	GOYES	cbCONT	no? continue
	A=A+A	A	shift graphic
	C=C-1	A	decrement # of shifts
	GOTO	cbSHIFT	do more

cbCONT	C=DAT0	3	get screen
	C=C&A	A	use mask
	GOSUB	cbFASTSTO_C	store data

	LCHEX	#00022	1 line distance
	AD0EX		get D0
	A=A+C	A	down 1 line
	D0=A		set D0
	D1=D1+	3	set D1 to next mask line
	D=D-1	A	dec counter
	GONC	cbNEXT_LINE	CARRY clear? do more

cbCHECK	ST=0	0	clear bounce flag
	LCHEX	#00004	loop 5 times
	D=C	A	Put it in D
cbGET_DATA	GOSUB	cbFASTRCL_C	recall data
	?C=0	A	no hit?
	GOYES	cbDEC	check next
	GOSUB	boBOUNCE	bounce it!
cbDEC	D=D-1	A	dec counter
	GONC	cbGET_DATA	do another line

	?ST=0	0	no bounce?
	RTNYES		RETURN
	ST=1	1	this tells (me) there was a recent bounce
	C=R3.F	A	get new direction
	GOSUBL	srSTO2_C	make it the current direction
	GOSUB	hbHIT_BRICK	check if a brick was hit
	?ST=1	4	did you die?
	RTNYES
	?ST=1	5	did you win?
	RTNYES
	GOTO	mvMOVE	go move it.


boBOUNCE	R4=C.F	A	save bounce data
	ST=1	0	set bounce flag
	C=0	A	clear C
	GOSUBL	srRCL2_A	direction
	?ABIT=1	0	ball going up?
	GOYES	boCHULFT	yes? check up-left
	GOTO	boCHDOWN	no? -> check down
boCHULFT	?ABIT=1	2	going left?
	GOYES	boUPLEFT	yes? do up-left
	?ABIT=1	3	going right?
	GOYES	boUPRIGHT	yes? do up-right

boUP	LCHEX	#4
	?C#D	A
	GOYES	boU_N1
	GOTO	boDOWN_DIR

boU_N1	GOSUB	boFIX_DATA
	?CBIT=0	0
	GOYES	boUN
	GOSUB	boDOWN_DIR
	GOTO	boRIGHT_DIR

boUN	?CBIT=0	4
	RTNYES
	GOSUB	boDOWN_DIR
	GOTO	boLEFT_DIR

boUPLEFT	LCHEX	#4
	?C#D	A	is D not equal to 4?
	GOYES	boUL_N1
	GOTO	boDOWN_DIR	set temp dir to down

boUL_N1	LCHEX	#3
	?C#D	A	is D not equal to 3?
	GOYES	boUL_N2
	GOSUB	boFIX_DATA	shift data back
	?CBIT=0	1	no down-right?
	GOYES	boULN	check next
	GOSUB	boDOWN_DIR	toggle temp-dir to down
	GOTO	boRIGHT_DIR	then to right

boULN	?CBIT=0	0	no bounce right?
	RTNYES		return
	GOTO	boRIGHT_DIR	toggle temp-dir to right

boUL_N2	?D=0	A	D=0?
	RTNYES		yes? return
	GOTO	boRIGHT_DIR	toggle temp-dir to right

boUPRIGHT	LCHEX	#4
	?C#D	A
	GOYES	boUR_N1
	GOTO	boDOWN_DIR

boUR_N1	LCHEX	#3
	?C#D	A
	GOYES	boUR_N2
	GOSUB	boFIX_DATA
	?CBIT=0	3	no dn-left?
	GOYES	boURN
	GOSUB	boDOWN_DIR
	GOSUB	boLEFT_DIR

boURN	?CBIT=0	4
	RTNYES
	GOTO	boLEFT_DIR

boUR_N2	?D=0	A
	RTNYES
	GOTO	boLEFT_DIR

boCHDOWN	?ABIT=0	1	ball not going down?
	RTNYES		must not be moving
	?ABIT=1	2	going left?
	GOYES	boDNLEFT	do down-left
	?ABIT=1	3	going right?
	GOYES	boDNRIGHT	do down-right

boDOWN	?D#0	A
	GOYES	boD_N1
	GOTO	boUP_DIR
boD_N1	GOSUB	boFIX_DATA
	?CBIT=0	0
	GOYES	boDN
	GOSUB	boUP_DIR
	GOTO	boRIGHT_DIR
boDN	?CBIT=0	4
	RTNYES
	GOSUB	boUP_DIR
	GOTO	boLEFT_DIR

boDNLEFT	?D#0	A
	GOYES	boDL_N1
	GOTO	boUP_DIR

boDL_N1	LCHEX	#1
	?D=C	A
	GOYES	boDL_N2
	GOTO	boRIGHT_DIR

boDL_N2	GOSUB	boFIX_DATA
	?CBIT=0	1	no up-right?
	GOYES	boDLN
	GOSUB	boUP_DIR
	GOTO	boRIGHT_DIR

boDLN	?CBIT=0	0	no right?
	RTNYES
	GOTO	boRIGHT_DIR

boDNRIGHT	?D#0	A
	GOYES	boDR_N1
	GOTO	boUP_DIR

boDR_N1	LCHEX	#1
	?D=C	A
	GOYES	boDR_N2
	GOTO	boLEFT_DIR

boDR_N2	GOSUB	boFIX_DATA
	?CBIT=0	3	no up-left?
	GOYES	boDRN
	GOSUB	boUP_DIR
	GOTO	boLEFT_DIR

boDRN	?CBIT=0	4	no left?
	RTNYES
	GOTO	boLEFT_DIR

boUP_DIR	A=R3.F	A	temp dir
	LCHEX	#C	binary -> 1100
	A=A&C	A	mask off other dir
	LCHEX	#00001	up
	A=A!C	A	or in onto dir
	R3=A.F	A	store it
	RTN

boDOWN_DIR	A=R3.F	A
	LCHEX	#C	binary -> 1100
	A=A&C	A
	LCHEX	#00002	down
	A=A!C	A
	R3=A.F	A
	RTN

boLEFT_DIR	A=R3.F	A
	LCHEX	#3	binary -> 0011
	A=A&C	A
	LCHEX	#00004	left
	A=A!C	A
	R3=A.F	A
	RTN

boRIGHT_DIR	A=R3.F	A
	LCHEX	#3	binary -> 0011
	A=A&C	A
	LCHEX	#00008	right
	A=A!C	A
	R3=A.F	A
	RTN

cbFASTSTO_C	CD0EX
	RSTK=C
	C=D	A
	C=C+CON	A,10
	A=C	A
	C=C+C	A
	C=C+C	A
	C=C+A	A
	A=R0.F	A
	A=A+C	A
	CD0EX
	D0=A
	DAT0=C	A
	C=RSTK
	D0=C
	RTN

cbFASTRCL_C	C=D	A
	C=C+CON	A,10
	A=C	A
	C=C+C	A
	C=C+C	A
	C=C+A	A
	A=R0.F	A
	A=A+C	A
	D0=A
	C=DAT0	A
	RTN

boFIX_DATA	C=R4.F	A
	A=B	A	# of shifts
boSHIFT	?A=0	A	do some?
	RTNYES		no? RTN
	CSRB.F	A	shift RIGHT
	A=A-1	A
	GOTO	boSHIFT

* ----------------------------------------------
* | hbHIT_BRICK
* | Find what kind of brick was hit, and
* | determine what to do about it. (and do it!)
* |
* | R1, R2, R3 are temporarily used.
* |
* ----------------------------------------------


hbHIT_BRICK	GOSUBL	srRCL3_A
	D1=A
	LCHEX	#00004	(4+1) times, you know.
	D=C	A
	GOSUB	cbFASTRCL_C
	?C=0	A	was there NO hit on this line?
	GOYES	hbCHECK0
	GOSUB	hbOUT
	?ST=1	4	did you die?
	RTNYES
	GOTO	hbCHECK1	only need to check 1 (since we hit)

hbCHECK0	GOSUBL	srRCL3_A
	LCHEX	#00088
	A=A+C	A
	D1=A
	D=0	A
	GOSUB	cbFASTRCL_C
	?C=0	A	no hit here?
	GOYES	hbCHECK3	then check sides
	GOSUB	hbOUT
	?ST=1	4	did you die?
	RTNYES

	GOSUBL	srRCL3_A
	LCHEX	#00022
	A=A+C	A
	D1=A
	LCHEX	#00003
	D=C	A
	GOSUB	cbFASTRCL_C
	?C=0	A
	RTNYES
	GOTO	hbOUT

hbCHECK3	GOSUBL	srRCL3_A
	LCHEX	#00022
	A=A+C	A
	D1=A
	LCHEX	#00003
	D=C	A
	GOSUB	cbFASTRCL_C
	?C=0	A
	GOYES	hbCHECK2
	GOTO	hbOUT

hbCHECK2	GOSUBL	srRCL3_A
	LCHEX	#00044
	A=A+C	A
	D1=A
	LCHEX	#00002
	D=C	A
	GOSUB	cbFASTRCL_C
	?C=0	A
	GOYES	hbCHECK1
	GOTO	hbOUT

hbCHECK1	GOSUBL	srRCL3_A
	LCHEX	#00066
	A=A+C	A
	D1=A
	LCHEX	#00001
	D=C	A
	GOSUB	cbFASTRCL_C
	?C=0	A
	RTNYES

hbOUT	LAHEX	#0000F	in binary 000000001111
	A=A&C	A	mask off nibble 0 (left-most on screen)
	?A#0	A	was there a hit here?
	GOYES	hbHIT
	D1=D1+	1	next nibble over
	LAHEX	#F0	in binary 000011110000
	A=A&C	A	mask off nibble 1 (middle nibble)
	?A#0	A	was there a hit here?
	GOYES	hbHIT
	D1=D1+	1

* I am assuming if you get this far with no hit,
* then the hit was in nibble 2.  (Unless you jumped to hbHIT)

hbHIT	C=0	W	zero reg
	A=0	W	zero reg
	GOSUBL	srRCL9_A	get screen address
	LCHEX	#45	offset to playing board
	C=C+A	A	add the amount
	AD1EX		get collision nibble
	A=A-C	A	get nibble offset
	GONC	hbNEXT1	carry is clear, OK for now
	GOTO	hbWALL	carry was set, we hit a wall.

hbNEXT1	LCHEX	#00022
	GOSUBL	DIV	A=A/C & C=A/C & B=remainder. (fields W)
	R1=C.F	A	save result
	A=B	A	get remainder
	LCHEX	#00003	each brick has 3 nibbles
	GOSUBL	DIV	C now contains column #
	LAHEX	#00007	7+1 columns
	?C<=A	A	column ok?
	GOYES	hbNEXT2
	GOTO	hbWALL	column too big? we hit a wall.

hbNEXT2	R3=C.F	A	put column # here for use in hbERS_BRICK
	LAHEX	#0000A	10 rows of bricks (use rows for offset)
	GOSBVL	=MUL#	B contains column offset
	C=B	A	column offset
	R2=C.F	A	save it

	A=R1.F	A	get earlier result back
	LCHEX	#00006	each brick has 6 rows
	GOSUBL	DIV	A & C contain row offset
	LCHEX	#00009	9+1 rows
	?C>=A	A	row ok?
	GOYES	hbNEXT3
	GOTO	hbWALL	row too big? we hit a wall

hbNEXT3	R1=A.F	A	R1 now contains row # for hbERS_BRICK

	C=R2.F	A	get column offset
	C=C+A	A	C is offset into brick matrix

* Now we can use C to see what kind of brick is at the collision site
* R1 == row#, R3 == column#

	GOSUBL	srRCL6_A	@ of current level brick matrix
	A=A+C	A
	D0=A
	C=0	A
	C=DAT0	1	get the brick
	?C#0	A	is it a brick?
	GOYES	hbNEXT4	then deal with it.
	GOTO	hbWALL	if there's no brick, it must be a wall

hbNEXT4	GOSUBL	srRCL15_A	erase mode
	?A=C	A	same as current brick?
	GOYES	hbNEXT5	deal with it.
	?ST=1	6	can we knock out diamonds?
	GOYES	hbCHECK_DM
	GOTO	hbCHK_OTHERS	otherwise, check others
hbCHECK_DM	LAHEX	#00002	a diamond brick
	?A=C	A	did you hit a diamond?
	GOYES	hbERASE_DIA	then deal with it
	GOTO	hbCHK_OTHERS	otherwise, deal with it differently

hbNEXT5	GOSUBL	srRCL16_A	number of bricks
	A=A-1	A	one less now
	GOSUBL	srSTO16_A
	?A=0	A	still more?
	GOYES	hbSET_ED	nope? set to erase diamonds
	GOSUB	hbERS_BRICK	erase the brick
	LCHEX	#00003	3 points for a NORMAL brick
	GOSUB	scADD_SCORE	add and display score.
	GOLONG	soERS_BRICK	make the sound
hbSET_ED	ST=1	6	we can erase diamonds now.
	GOSUB	hbERS_BRICK	we still need to erase that brick
	LCHEX	#00003	3 points for a NORMAL brick
	GOSUB	scADD_SCORE	add and display score.
	GOLONG	soERS_BRICK	make the sound

hbERASE_DIA	GOSUBL	srRCL17_A	number of diamonds left
	A=A-1	A	one less
	GOSUBL	srSTO17_A	store it

	?A#0	A	still more?
	GOYES	hbER_CONT
	ST=1	5	set YOU WON flag to 1

hbER_CONT	GOSUB	hbERS_BRICK	erase the diamond
	?ST=0	11	sound on?
	GOYES	hbED_START	yes? make the sound.

	LCHEX	#000E2	length of sound (ticks)
	GOSUBL	udDELAY	delay
	GOTO	hbED_END

hbED_START	LCHEX	#00066	starting pitch
	D=C	A

hbED_AGAIN	LCHEX	#800
	OUT=C
	C=D	A	current pitch
hbED_DEC1	C=C-1	A
	GONC	hbED_DEC1
	C=0	A
	OUT=C
	LCHEX	#00066
	C=C-D	A	inverse of current pitch
hbED_DEC2	C=C-1	A
	GONC	hbED_DEC2
	D=D-1	A
	LCHEX	#00050	ending value
	?D>C	A
	GOYES	hbED_AGAIN

hbED_END	LCHEX	#00100	100 points for a diamond (in decimal)
	GOTO	scADD_SCORE	add and display score.


hbCHK_OTHERS	?CBIT=0	3	check bit 3
	GOYES	hbCO_2
	?CBIT=1	0	if this bit is 0, then act as wall
	GOYES	hbCO_1	must be an erase mode changer
	GOTO	hbWALL	must be a brick you can't erase
hbCO_1	GOSUBL	srRCL15_A	get current erase mode
	C=C-1	A	one less than brick# is erase mode
	?C#A	A	is it a new mode?
	GOYES	hbCO_1CONT1	yes? change it
	GOTO	hbWALL	otherwise, act as wall
hbCO_1CONT1	GOSUBL	srSTO15_C	set new mode
	GOSUB	emDRAW_BRICK	draw new brick
	GOLONG	soEM_CHANGE	do erase mode change sound (then RTN)

hbCO_2	?CBIT=1	2	check bit 2
	GOYES	hbCO_3
	GOTO	hbWALL
* the above goto will only happen if the brick is
* 1(first brick) 2(diamond) or 3(wall) and the erase
* mode is not 1 or 2.  So, they all act as walls here.

hbCO_3	?CBIT=1	1	check bit 1
	GOYES	hbCO_4

	?CBIT=0	0	check bit 0
	GOYES	hbCO_KEY

* following is if you just hit a lock brick

	?ST=1	7	do you have a key?
	GOYES	hbCOL_OK	ok!
	GOTO	hbWALL	if not, be a wall.
hbCOL_OK	ST=0	7	no more key
	GOSUB	hbERS_BRICK	erase the lock brick
	GOSUB	hbDRAW_KEY	erase the key
	GOSUBL	soERASE_LOCK	make the sound
	LCHEX	#00050	50 points for a lock brick
	GOTO	scADD_SCORE	add and display score.

* following is if you just hit a key brick

hbCO_KEY	?ST=0	7	do you not have a key?
	GOYES	hbCOK_OK	ok!
	GOTO	hbWALL	can't have more than one
hbCOK_OK	ST=1	7	now you have a key
	GOSUB	hbERS_BRICK	erase the brick
	GOSUB	hbDRAW_KEY	draw the key
	GOSUBL	soERASE_KEY	make the sound
	LCHEX	#00025	25 points for a key brick
	GOTO	scADD_SCORE	add and display score.

hbCO_4	?CBIT=1	0	check bit 0
	GOYES	hbCO_5
	GOTO	hbDEAD	you hit a cross!!

* To get here (below) you hit a double arrow brick.
hbCO_5	?ST=0	3	is direction NORMAL?
	GOYES	hbDIRCH1
	ST=0	3	dir is now NORMAL
	GOSUB	hbERS_BRICK	erase the arrows brick
	GOLONG	soDOUBLE_ARR	make a sound
hbDIRCH1	ST=1	3	dir is now opposite
	GOSUB	hbERS_BRICK
	GOLONG	soDOUBLE_ARR	make a sound

hbERS_BRICK	A=R1.F	A	row number
	LCHEX	#000CC	(6 * #22h) (6 lines per brick)
	GOSBVL	=MUL#	result in B
	A=B	A
	R1=A.F	A
	A=R3.F	A	column number
	LCHEX	#03	(3 nibbles per brick)
	GOSBVL	=MUL#	result in B
	C=B	A
	A=R1.F	A
	A=A+C	A	now I have the (almost) correct nibble offset
	LCHEX	#00045	two lines down, one nibble over
	A=A+C	A	now I have the correct nibble offset
	GOSUBL	srRCL9_C	screen bitmap address
	A=A+C	A	now I have the address of the brick nibble
	D0=A		into D0 (of course)

	LCHEX	#00005	(5 + 1) lines per brick
	B=C	A
	LCHEX	#22	same ole thing
	D=C	A
	LAHEX	#000	3 zero nibbles per line
hbERASE	DAT0=A	3	zap one line
	CD0EX
	C=C+D	A
	D0=C
	B=B-1	A
	GONC	hbERASE
	RTN

hbWALL	?ST=0	11	is sound on?
	GOYES	hbW_START	yes? make the sound.

	LCHEX	#0005E	length of sound (ticks)
	GOLONG	udDELAY	delay (RTN)

hbW_START	LCHEX	#00008	note length
	D=C	A

hbW_AGAIN	LCHEX	#800
	OUT=C
	LCHEX	#00050	pitch #1
hbW_DEC1	C=C-1	A
	GONC	hbW_DEC1
	C=0	A
	OUT=C
	LCHEX	#00028	pitch #2
hbW_DEC2	C=C-1	A
	GONC	hbW_DEC2
	D=D-1	A
	GONC	hbW_AGAIN
	RTN


hbDEAD	C=0	A
	GOSUBL	srSTO2_C	set dir to none
	ST=1	10	set to erase (for db)
	GOSUBL	mvMOVE	erase the ball
	ST=0	10	set to NORMAL (for db)

	GOSUBL	srRCL43_C	look in crystal ball...
	?C#0	A	is something in crytal ball?
	GOYES	hbD_SCHECK	start rumors about myths
	GOSUBL	srRCL22_C	number of lives left
	C=C-1	A	one less now
	GOSUBL	srSTO22_C	store it
	?C#0	A	still some lives left?
	GOYES	hbD_SCHECK	go check sound
	GOSUBL	grGAME_OVER	display GAME OVER banner

hbD_SCHECK	?ST=0	11	is sound on?
	GOYES	hbD_SOUND	make the sound

	LCHEX	#03000
	B=C	A
hbD_CKEY	LCHEX	#1FF	| wait here until
	OUT=C		| they stop pressing
	GOSBVL	=CINRTN	| a key
	LAHEX	#0003F	|
	A=A&C	A	|
	?A=0	A	|
	GOYES	hbD_NO_KEY	|
	B=B-1	A	don't wait forever, though
	GONC	hbD_CKEY
	GOTO	hbDEND

hbD_NO_KEY	LCHEX	#000CC	about 1 second
	GOSUBL	waWAIT	delay, pressing a key gets out
	GOTO	hbDEND

hbD_SOUND	LCHEX	#00090	starting pitch
	D=C	A

hbD_START	LCHEX	#00002	length
	B=C	A

hbD_AGAIN	LCHEX	#800
	OUT=C
	C=D	A	current pitch
hbD_DEC1	C=C-1	A
	GONC	hbD_DEC1
	C=0	A
	OUT=C
	LCHEX	#00020	a pitch
hbD_DEC2	C=C-1	A
	GONC	hbD_DEC2
	B=B-1	A
	GONC	hbD_AGAIN

	D=D+CON	A,2
	LCHEX	#00100
	?C#D	A
	GOYES	hbD_START

hbDEND	ST=1	4
	RTN

hbDRAW_KEY	C=PC
	GOTO	hbDK_NEXT

	NIBHEX	C10	|Key graphic
	NIBHEX	2E7	|
	NIBHEX	225	|Data
	NIBHEX	C15	|

hbDK_NEXT	C=C+CON	A,4	C is now @ of key graphic
	D1=C		put it into D1

	GOSUBL	srRCL9_A	get screen @
	LCHEX	#00508	{#78h #25h}
	A=A+C	A	add offset...
	D0=A		...and into D0

	LCHEX	#00003	(3+1) lines
	D=C	A
	LCHEX	#22	typical
	B=C	A

hbDK_LINE	C=DAT1	3	get one line of graphic
	?ST=1	7	do you have a key?
	GOYES	hbDK_OK	ok, draw it
	C=0	A	otherwise, blank
hbDK_OK	DAT0=C	3	draw it
	CD0EX
	C=C+B	A	down one line
	D0=C
	D1=D1+	3	next graphic line
	D=D-1	A	done?
	GONC	hbDK_LINE	do some more
	RTN

* ----------------------------------------
* | For emDRAW_BRICK, I assume that the
* | brick # to be drawn is contained in C,
* | field A.  And the address of the
* | brick graphics is in sr5.
* ----------------------------------------
emDRAW_BRICK	B=C	A	put brick# in B
	B=B-1	A	this is so GONC will work.
	GOSUBL	srRCL5_C	get block data addr
emADD	C=C+CON	A,15	|each brick is 18
	C=C+CON	A,3	|nibbles of data (6*3)
	B=B-1	A
	GONC	emADD
	D1=C

	GOSUBL	srRCL9_A	screen bitmap @
	LCHEX	#004E2	this puts us at {#68h #24h}
	C=C+A	A
	D0=C

	LCHEX	#00005	(5+1) times
	B=C	A
	LCHEX	#22
emLINE	A=DAT1	3
	DAT0=A	3
	D1=D1+	3
	AD0EX
	A=A+C	A
	D0=A
	B=B-1	A
	GONC	emLINE
	RTN

* ----------------------------------------
* |
* | These are the score adding and
* | displaying routines
* |
* ----------------------------------------

* for scADD_SCORE the amount to add to the score
* (in Var #23) must be contained in C, field A. (IN DECIMAL)

scADD_SCORE	GOSUBL	srRCL23_A	get current score
	SETDEC
	C=C+A	A
	SETHEX
	GOSUBL	srSTO23_C	save it

* for scDISP_SCORE the score to be displayed must
* be contained in C, field A.

scDISP_SCORE	R2=C.F	A	put the score here
	C=PC		get curr addr.
	GOTO	scNEXT

* The following data is stored in a strange and complicated
* way.  The reason for storing the data this way is to
* facilitate as well as accellerate the use of it.
*
* scD1L1 stands for (SCORE, Digit 1, Line 1)
* Digit 1 is the digit that fits right on a nibble boundary,
* Digit 2 is shifted left one bit (as pixels on the screen)
* Digit 3 is shifted left two bits
* Digit 4 is shifted left three bits
*
* Each line of each digit contains 2 nibbles of data for
* each digit (0-9) (With the exception of digit 1, which
* only contains 1 nibble per digit, since the entire
* digit can fit in one nibble, on the boundary).
*
* The score drawing routine must draw the digits one line
* at a time, so each digit type is grouped by line numbers.
* The drawing routine first gets a digit of type 1, line 1
* (right most digit in score), then a digit of type 2, line 1
* then type 3, line 1, and type 4, line 1.  Now it has created a
* one-line graphic of the score that it displays.  Then it
* does line 2 in a similar fashion, then line 3, etc.
* The left most digit of the score is on a nibble boundary, and
* is treated special.  Since the right most 4 digits can fit
* into a 5 nibble area (field A) they are treated separately.
* The left most digit will not change very often, so each time
* this drawing routine is called, if the digit hasn't changed,
* then it is left as-is.  This will speed up the scoring process.

	NIBHEX	646F9FEF66	scD1L1
	NIBHEX	03020387848707870303	scD2L1
	NIBHEX	810181C342C383C38181	scD3L1
	NIBHEX	C080C0E121E1C1E1C0C0	scD4L1

	NIBHEX	9698911899	scD1L2
	NIBHEX	84038404848080048484	scD2L2
	NIBHEX	42814202424040024242	scD3L2
	NIBHEX	21C02101212020012121	scD4L2

	NIBHEX	9484977469	scD1L3
	NIBHEX	84020402848383020384	scD2L3
	NIBHEX	4201020142C1C1018142	scD3L3
	NIBHEX	2180018021E0E080C021	scD4L3

	NIBHEX	9468F8929E	scD1L4
	NIBHEX	84020304870484018407	scD2L4
	NIBHEX	42018102C30242804283	scD3L4
	NIBHEX	2180C001E101214021C1	scD4L4

	NIBHEX	9419899298	scD1L5
	NIBHEX	84028084048484018404	scD2L5
	NIBHEX	42014042024242804202	scD3L5
	NIBHEX	21802021012121402101	scD4L5

	NIBHEX	6EF6866267	scD1L6
	NIBHEX	03078703040303010383	scD2L6
	NIBHEX	8183C3810281818081C1	scD3L6
	NIBHEX	C0C1E1C001C0C040C0E0	scD4L6

scNEXT	C=C+CON	A,4	C is now addr of data
	R4=C.F	A	temp storage
	R1=C.F	A	for use in scLEFT_DIGIT

	GOSUBL	srRCL9_A	screen @
	LCHEX	#0005F	{#6Ch #2h}
	A=A+C	A	add it
	D0=A		into D0

	LCHEX	#00005	5 + 1 lines per digit
	D=C	A	put in D.

scLINE	C=R2.F	A	the score
	B=C	A	put it in B
	A=R4.F	A	get digit data @

	LCHEX	#0000F	mask
	C=C&B	A	mask off nibble 0 of score
	A=A+C	A	add offset in @ of data
	D1=A		put @ here
	A=0	A
	A=DAT1	1	get 1 nibble
	ASL	A	| shift 2 nibbles
	ASL	A	| to the left
	R3=A.F	A	temp storage for graphic
	A=R4.F	A	get digit data @
	A=A+CON	A,10	move to next digit type
	R4=A.F	A	store it
	BSR	A	shift B right 1 nibble

	LCHEX	#0000F	mask
	C=C&B	A	mask off nibble 1 of score
	A=A+C	A	|add offset
	A=A+C	A	|(2 nibbles, we add twice)
	D1=A		put @ here
	A=0	A
	A=DAT1	2	get 2 nibbles
	C=R3.F	A	get graphic
	C=C!A	A	or on the new stuff
	CSL	A	shift only once this time
	R3=C.F	A	temp storage for graphic
	A=R4.F	A	get digit data @
	A=A+CON	A,10	| move to next digit type
	A=A+CON	A,10	| (2 nibbles, we add twice)
	R4=A.F	A	store it
	BSR	A	shift B right 1 nibble

	LCHEX	#0000F	mask
	C=C&B	A	mask off nibble 2 of score
	A=A+C	A
	A=A+C	A
	D1=A
	A=0	A
	A=DAT1	2
	C=R3.F	A
	C=C!A	A
	CSL	A
	R3=C.F	A
	A=R4.F	A	get digit data @
	A=A+CON	A,10	| move to next digit type
	A=A+CON	A,10	| (2 nibbles, we add twice)
	R4=A.F	A	store it
	BSR	A

	LCHEX	#0000F	mask
	C=C&B	A	mask off nibble 3 of score
	A=A+C	A
	A=A+C	A
	D1=A
	A=0	A
	A=DAT1	2
	C=R3.F	A
	C=C!A	A
	A=R4.F	A	get digit data @
	A=A+CON	A,10	| move to next digit type
	A=A+CON	A,10	| (2 nibbles, we add twice)
	R4=A.F	A	store it
	BSR	A

	DAT0=C	A	put it on the screen
	LCHEX	#00022	one line
	AD0EX		get D0
	A=A+C	A
	D0=A	A

	D=D-1	A	one line less to do...
	GOC	scLEFT_DIGIT	do the left digit
	GOTO	scLINE	go do more

scLEFT_DIGIT	GOSUBL	srRCL9_A	screen address
	LCHEX	#0005E	{#68h #2h}
	A=A+C	A
	D0=A		set D0

	A=R1.F	A	@ of digit data
	LCHEX	#0000F	mask
	C=C&B	A	get the left-most digit
	A=A+C	A	offset to proper digit
	D1=A		put it into D1

	LCHEX	#00005	5+1 lines
	D=C	A
	LCHEX	#00022	one line on screen
	B=C	A
	LCHEX	#00046	offset to next digit line

scLD_LINE	A=DAT1	1	get 1 line of the digit
	DAT0=A	1	put it on the screen
	AD0EX		get D0
	A=A+B	A	down 1 line
	D0=A
	AD1EX		get D1
	A=A+C	A	add offset
	D1=A
	D=D-1	A
	GONC	scLD_LINE	do another

	A=R2.F	A	current score
	GOSUBL	srRCL26_C	next bonus life at this score
	?C>A	A	no extra life yet?
	RTNYES		end of sc

	LA(5)	BONUS_LIFE	number of points until next life
	SETDEC
	C=C+A	A	add the increment (in decimal)
	SETHEX
	?C<A	A	have we gone too far?
	GOYES	scBONUS_FIX
	GOTO	scBONUS_STO
scBONUS_FIX	LCHEX	#99999	max bonus score
scBONUS_STO	GOSUBL	srSTO26_C	store it for next extra life.

	GOSUBL	srRCL22_C	number of lives left
	C=C+1	A	add one
	GOSUBL	srSTO22_C	store it
	GOSUBL	dlDRAW_LIVES	display the change
	GOTO	soBONUS_LIFE	make a nifty sound, auto RTN

deDRAW_LEVEL	C=PC
	GOTO	deNEXT

* This data here is the same as in (sc) except that there are
* only digit types 3 and 4.

	NIBHEX	810181C342C383C38181	deD3L1
	NIBHEX	C080C0E121E1C1E1C0C0	deD4L1

	NIBHEX	42814202424040024242	deD3L2
	NIBHEX	21C02101212020012121	deD4L2

	NIBHEX	4201020142C1C1018142	deD3L3
	NIBHEX	2180018021E0E080C021	deD4L3

	NIBHEX	42018102C30242804283	deD3L4
	NIBHEX	2180C001E101214021C1	deD4L4

	NIBHEX	42014042024242804202	deD3L5
	NIBHEX	21802021012121402101	deD4L5

	NIBHEX	8183C3810281818081C1	deD3L6
	NIBHEX	C0C1E1C001C0C040C0E0	deD4L6

deNEXT	C=C+CON	A,4
	R4=C		save @ of numbers

	GOSUBL	srRCL20_C	(lev #) in hex
	A=0	A
deCONV	SETDEC
	A=A+1	A	add one in decimal
	SETHEX
	C=C-1	A	subtract one in hex
	GONC	deCONV	do more (for small nums, this works great)
	R2=A.F	A	(lev #)+1 in decimal

	GOSUBL	srRCL9_A	screen @
	LCHEX	#001B6	{#78h #Ch}
	A=A+C	A
	D0=A

	LCHEX	#00005	(5+1) lines in a digit
	D=C	A

deLINE	C=R2.F	A	lev #
	B=C	A
	A=R4.F	A	get digit data @
	LCHEX	#0000F	mask
	C=C&B	A	mask off nibble 0 of level num.
	A=A+C	A	|add offset into digit data
	A=A+C	A	| (twice, each digit is 2 nibbles)
	D1=A
	A=0	A
	A=DAT1	2	get 2 nibbles of data
	ASL	A	shift it left 1 nibble
	R3=A.F	A	store the graphic
	A=R4.F	A	data addr
	A=A+CON	A,10	| increment @ pointer
	A=A+CON	A,10	| twice, for the two nibbles
	R4=A.F	A	save it
	BSR	A	shift B right to get next digit

	LCHEX	#0000F	mask
	C=C&B	A	mask off nibble 1 of lev num
	A=A+C	A	| add offset into digit data
	A=A+C	A	| (twice, again)
	D1=A		put addr here
	A=0	A
	A=DAT1	2	get two nibbles data
	C=R3.F	A	get back graphic
	C=C!A	A	or new data on
	A=R4.F	A	digit data @
	A=A+CON	A,10	| increment @ pointer
	A=A+CON	A,10	| twice, for the two nibbles
	R4=A.F	A

	DAT0=C	3	put the 3 nibbles onto the screen
	LCHEX	#00022
	AD0EX
	A=A+C	A	down one line
	D0=A

	D=D-1	A
	RTNC		if carry==1, return (we're done)
	GOTO	deLINE	otherwise, carry on...


soEM_CHANGE	?ST=0	11	is sound on?
	GOYES	so3_START	yes? make the sound.

	LCHEX	#000E2	length of sound (ticks)
	GOTO	udDELAY	delay (RTN)

so3_START	LCHEX	#00010	note length
	D=C	A

so3_AGAIN	LCHEX	#800
	OUT=C
	LCHEX	#00070	pitch #1
so3_DEC1	C=C-1	A
	GONC	so3_DEC1
	C=0	A
	OUT=C
	LCHEX	#00020	pitch #2
so3_DEC2	C=C-1	A
	GONC	so3_DEC2
	D=D-1	A
	GONC	so3_AGAIN
	RTN

soERS_BRICK	?ST=0	11	is sound on?
	GOYES	so2_START	yes? make the sound.

	LCHEX	#00042	length of sound (ticks)
	GOTO	udDELAY	delay (RTN)

so2_START	LCHEX	#00007	note length
	D=C	A

so2_AGAIN	LCHEX	#800
	OUT=C
	LCHEX	#00020	pitch #1
so2_DEC1	C=C-1	A
	GONC	so2_DEC1
	C=0	A
	OUT=C
	LCHEX	#00040	pitch #2
so2_DEC2	C=C-1	A
	GONC	so2_DEC2
	D=D-1	A
	GONC	so2_AGAIN
	RTN

soDOUBLE_ARR	?ST=0	11	is sound on?
	GOYES	soDA_FIRST	yes? make the sound.

	LCHEX	#00042	length of sound (ticks)
	GOTO	udDELAY	delay (RTN)

soDA_FIRST	LCHEX	#00080	starting pitch
	D=C	A

soDA_START	LCHEX	#00002	length
	B=C	A

soDA_AGAIN	LCHEX	#800
	OUT=C
	C=D	A	current pitch
soDA_DEC1	C=C-1	A
	GONC	soDA_DEC1
	C=0	A
	OUT=C
	LCHEX	#00010	a pitch
soDA_DEC2	C=C-1	A
	GONC	soDA_DEC2
	B=B-1	A
	GONC	soDA_AGAIN

	D=D-CON	A,4
	LCHEX	#00050	ending pitch
	?D>C	A
	GOYES	soDA_START

	RTN		done with soDOUBLE_ARR


soERASE_KEY	?ST=0	11	is sound on?
	GOYES	soEK_FIRST	yes? make the sound.

	LCHEX	#00042	length of sound (ticks)
	GOTO	udDELAY	delay (RTN)

soEK_FIRST	LCHEX	#00021	starting pitch
	D=C	A

soEK_START	LCHEX	#00003	length
	B=C	A

soEK_AGAIN	LCHEX	#800
	OUT=C
	C=D	A	current pitch
soEK_DEC1	C=C-1	A
	GONC	soEK_DEC1
	C=0	A
	OUT=C
	C=D	A	current pitch
soEK_DEC2	C=C-1	A
	GONC	soEK_DEC2
	B=B-1	A
	GONC	soEK_AGAIN

	LCHEX	#00030
	?D<C	A
	GOYES	soEK_ADD

	D=D-CON	A,15
	GOTO	soEK_START

soEK_ADD	D=D+CON	A,16
	LCHEX	#00034
	?D#C	A
	GOYES	soEK_START

	RTN		end of soERASE_KEY


soERASE_LOCK	?ST=0	11	is sound on?
	GOYES	soEL_FIRST	yes? make the sound.

	LCHEX	#00042	length of sound (ticks)
	GOTO	udDELAY	delay (RTN)

soEL_FIRST	LCHEX	#00090	starting pitch
	D=C	A

soEL_START	LCHEX	#00008	length
	B=C	A

soEL_AGAIN	LCHEX	#800
	OUT=C
	C=D	A	current pitch
soEL_DEC1	C=C-1	A
	GONC	soEL_DEC1
	C=0	A
	OUT=C
	C=D	A	current pitch
soEL_DEC2	C=C-1	A
	GONC	soEL_DEC2
	B=B-1	A
	GONC	soEL_AGAIN

	LCHEX	#00090
	?D<C	A
	GOYES	soEL_ADD

	LCHEX	#00050
	D=D-C	A
	LCHEX	#00041
	?D=C	A
	RTNYES		end of soERASE_LOCK
	GOTO	soEL_START

soEL_ADD	LCHEX	#00051
	D=D+C	A
	GOTO	soEL_START


soBONUS	?ST=0	11	is sound on?
	GOYES	soB_START	yes? make the sound.

	LCHEX	#000D7	length of sound (ticks)
	GOTO	udDELAY	delay (RTN)

soB_START	LCHEX	#00010	note length
	D=C	A
	LCHEX	#00060
	B=C	A

soB_AGAIN	LCHEX	#800
	OUT=C
	C=B	A	pitch #1
soB_DEC1	C=C-1	A
	GONC	soB_DEC1
	C=0	A
	OUT=C
	C=B	A	#1
	LAHEX	#00020
	C=C-A	A	pitch #2
soB_DEC2	C=C-1	A
	GONC	soB_DEC2
	B=B-CON	A,2
	D=D-1	A
	GONC	soB_AGAIN
	RTN

soBONUS_LIFE	?ST=1	11	sound off?
	RTNYES		then all is quiet (no delay)

	LCHEX	#00015	starting pitch
	D=C	A
	A=0	A

soBL_START	LCHEX	#00003	length
	B=C	A

soBL_AGAIN	LCHEX	#800
	OUT=C
	C=D	A	current pitch
soBL_DEC1	C=C-1	A
	GONC	soBL_DEC1
	C=0	A
	OUT=C
	C=D	A	current pitch
soBL_DEC2	C=C-1	A
	GONC	soBL_DEC2
	B=B-1	A
	GONC	soBL_AGAIN

	?A#0	A
	GOYES	soBL_DOWN

	D=D-1	A
	?D#0	A
	GOYES	soBL_START

soBL_DOWN	LAHEX	#1
	D=D+1	A
	LCHEX	#00015
	?C#D	A
	GOYES	soBL_START
soBL_END	RTN


soHIGH_SCORE	?ST=1	11	is sound off?
	RTNYES		then we're done (no delay)

	LCHEX	#0004F	pitch
	B=C	A
	A=0	A	a flag
soHS_LENGTH	LCHEX	#00004	note length
	D=C	A

soHS_AGAIN	LCHEX	#800
	OUT=C
	C=B	A	pitch #1
soHS_DEC1	C=C-1	A
	GONC	soHS_DEC1
	C=0	A
	OUT=C
	C=B	A	pitch #2
soHS_DEC2	C=C-1	A
	GONC	soHS_DEC2
	D=D-1	A
	GONC	soHS_AGAIN

	?A#0	A
	GOYES	soHS_UP

	B=B-CON	A,10
	GONC	soHS_LENGTH
	LAHEX	#0004F
	B=B+CON	A,10
	B=B+CON	A,10
	GOTO	soHS_LENGTH

soHS_UP	B=B+CON	A,10
	?A>B	A
	GOYES	soHS_LENGTH
	RTN

soMENU_UP	?ST=1	11	is sound off?
	RTNYES		then we're done (no delay)

	LCHEX	#00060	starting pitch
	D=C	A

soMU_START	LCHEX	#00004	length
	B=C	A

soMU_AGAIN	LCHEX	#800
	OUT=C
	C=D	A	current pitch
soMU_DEC1	C=C-1	A
	GONC	soMU_DEC1
	C=0	A
	OUT=C
	C=D	A	current pitch
soMU_DEC2	C=C-1	A
	GONC	soMU_DEC2
	B=B-1	A
	GONC	soMU_AGAIN

	D=D-CON	A,4
	LCHEX	#00020	ending pitch
	?D>C	A
	GOYES	soMU_START
	RTN


soMENU_DOWN	?ST=1	11	is sound off?
	RTNYES		then we're done (no delay)

	LCHEX	#00020	starting pitch
	D=C	A

soMD_START	LCHEX	#00004	length
	B=C	A

soMD_AGAIN	LCHEX	#800
	OUT=C
	C=D	A	current pitch
soMD_DEC1	C=C-1	A
	GONC	soMD_DEC1
	C=0	A
	OUT=C
	C=D	A	current pitch
soMD_DEC2	C=C-1	A
	GONC	soMD_DEC2
	B=B-1	A
	GONC	soMD_AGAIN

	D=D+CON	A,4
	LCHEX	#00060	ending pitch
	?D<C	A
	GOYES	soMD_START
	RTN


soMYTH	?ST=1	11	is sound off?
	RTNYES		then we're done (no delay)

	LCHEX	#00021	starting pitch
	D=C	A
	A=0	A

soMY_START	LCHEX	#00003	length
	B=C	A

soMY_AGAIN	LCHEX	#800
	OUT=C
	C=D	A	current pitch
soMY_DEC1	C=C-1	A
	GONC	soMY_DEC1
	C=0	A
	OUT=C
	C=D	A	current pitch
soMY_DEC2	C=C-1	A
	GONC	soMY_DEC2
	B=B-1	A
	GONC	soMY_AGAIN

	LCHEX	#00030
	?D<C	A
	GOYES	soMY_ADD

	D=D-CON	A,14
	GOTO	soMY_START

soMY_ADD	D=D+CON	A,16
	A=A+1	A
	LCHEX	#00030
	?C>A	A
	GOYES	soMY_START

	RTN


* This new udDELAY routine bases its delays on the internal TIMER2.
* This is the way I should have done this all along, but now it's done.
* Send C, field A with the number of ticks (n) you would like to delay.
* One tick is 1/8192 of a second.

* Registers used:  C, R1, R2, R3, CARRY, ST(0)
* A and D1 are saved.

* If you have a pre-starting time, then you must first place n in R2
* then put the starting time in C(A) and call udDELAYcont.
* In this case, D1, and A are not preserved.

udDELAY	R2=C.F	A	n
	C=A	A	save A
	GOSBVL	=CSLW5
	CD1EX		save D1
	R3=C

	D1=(5)	=TIMER2
	C=0	A
	C=DAT1	4	read 4 nibbles
udDELAYcont	R1=C.F	A	starting time
	D1=(5)	=TIMER2	for the cases where udDELAYcont is used

udDELAYLP	LCHEX	#010
	OUT=C
	GOSBVL	=CINRTN
	?CBIT=0	3	[+/-] key
	GOYES	udNO_S
	ST=1	0	[+/-] is being pressed
	GOSUBL	srRCL28_C	get [+/-] flag
	?C#0	A	are you still pressing [+/-]
	GOYES	udNO_FLAG	yes? check [ON]
	?ST=0	11	is sound on?
	GOYES	udSET	yes? go turn it off
	ST=0	11	Turn sound on.

	LAHEX	#00020	length of blip
udSO_BLIP	LCHEX	#800
	OUT=C
	LCHEX	#00012	pitch 1
udSO_DEC1	C=C-1	A
	GONC	udSO_DEC1
	C=0	A
	OUT=C
	LCHEX	#0000E	pitch 2
udSO_DEC2	C=C-1	A
	GONC	udSO_DEC2
	A=A-1	A
	GONC	udSO_BLIP
	GOTO	udNEXT	go set ST0 to say you've pressed S already

udSET	ST=1	11	turn sound off.
udNEXT	LCHEX	1	make C non-NULL
	GOSUBL	srSTO28_C	set [+/-] flag to being pressed.
	GOTO	udNO_FLAG	skip next line

udNO_S	ST=0	0
udNO_FLAG	LCHEX	#400	[ON] key
	OUT=C
	GOSBVL	=CINRTN
	?CBIT=0	15	no press?
	GOYES	udNO_ON
	GOLONG	maEND	[ON] was pressed, you're done!

udNO_ON	C=0	A
	C=DAT1	4	read TIMER2
	A=R1.F	A	get start time
	C=A-C	A	elapsed time
	GOC	udFIX	did we have a carry? (wrap around)
udOK	A=R2.F	A	n
	?A<C	A	done with delay?
	GOYES	udDONE
	GOTO	udDELAYLP

udDONE	C=0	A
	C=DAT1	4	get TIMER2
	GOSUBL	srSTO47_C	save (me) new starting time

	?ST=1	0
	GOYES	udLEAVE_FLAG
	C=0	A
	GOSUBL	srSTO28_C	set [+/-] flag to not pressing.
udLEAVE_FLAG	C=R3
	D1=C		restore D1
	GOSBVL	=CSRW5
	A=C	A	restore A
	RTN		done with udDELAY

udFIX	LAHEX	#FE000	(#100000h - #2000h)
	C=C-A	A	fix wrap-around
	GOTO	udOK


dsDRAW_SCN	GOSUBL	srRCL6_C	get data addr.
	GOSUBL	srSTO4_C	put it here

	C=PC
	GOTO	dsNEXT2

	NIBHEX	000000000000000000	dsB0
	NIBHEX	EF7108108108108EF7	dsB1
	NIBHEX	E97F0F70E70EF0FE97	dsB2
	NIBHEX	EF7FFFB7FFEDFFFEF7	dsB3
	NIBHEX	EF73EFD18DDA3EAEF7	dsB4
	NIBHEX	EF7F9FF6F70E70EEF7	dsB5
	NIBHEX	EF7F9F30C30CF9FEF7	dsB6
	NIBHEX	6F73FE1CC3387FCEF6	dsB7
	NIBHEX	EF7108DFBDFB108EF7	dsB8
	NIBHEX	EF710FD7FD7F10FEF7	dsB9
	NIBHEX	EF7BAA55DBAA55DEF7	dsBA
	NIBHEX	EF7B2F55FB2F55FEF7	dsBB
	NIBHEX	EF759A59A59A59AEF7	dsBC
	NIBHEX	EF751F51F51F51FEF7	dsBD
	NIBHEX	EF799999976E76EEF7	dsBE
	NIBHEX	EF791F91F76F76FEF7	dsBF

dsNEXT2	C=C+CON	A,4	C is now @dsB1
	GOSUBL	srSTO5_C

	C=0	A
	GOSUBL	srSTO27_C	column counter
	GOSUBL	srSTO16_C	number of bricks
	GOSUBL	srSTO17_C	number of diamonds

dsDOCOLUMN	LCHEX	#00018	one line
	B=C	A
	GOSUBL	srRCL8_A	get STACK GROB @.

	LCHEX	#00008	max columns
	D=C	A
	GOSUBL	srRCL27_C	get column #
	?C#D	A	not done?
	GOYES	dsADDCOL	continue

	RTN		return from ds

dsADDCOL	?C=0	A
	GOYES	dsOK
	A=A+CON	A,3	move over one column.
	C=C-1	A
	GOTO	dsADDCOL

dsOK	D0=A

	GOSUBL	srRCL27_C	get col#
	C=C+1	A	add one for next time
	GOSUBL	srSTO27_C	store it.

	LCHEX	#00009	(9 + 1) times
	D=C	A	counter

dsDOBLOCK	GOSUBL	srRCL4_C	get board data addr
	D1=C		put it in D1
	C=C+1	A	next nibble
	GOSUBL	srSTO4_C	store it.

	C=0	A	zero reg
	C=DAT1	1	get a brick

	A=0	A
	LAHEX	#1	brick #1
	?C#A	A	not one of these?
	GOYES	dsNB_1
	GOSUBL	srRCL16_A
	A=A+1	A
	GOSUBL	srSTO16_A
dsNB_1	LAHEX	#2	a diamond
	?C#A	A	not one of these?
	GOYES	dsNB_2
	GOSUBL	srRCL17_A
	A=A+1	A
	GOSUBL	srSTO17_A
dsNB_2	?CBIT=0	3	|if bit 3=0 then
	GOYES	dsNB_3	|it's not a reg brick
	?CBIT=1	0	]if bit 0=0 then
	GOYES	dsNB_3	]it's a change-mode brick
	GOSUBL	srRCL16_A	}we know it's a reg brick
	A=A+1	A	}so we increment
	GOSUBL	srSTO16_A	}the number.

dsNB_3	B=C	A	put it here.
	GOSUBL	srRCL5_C	get block data addr
dsADD	?B=0	A	no block?
	GOYES	dsNOADD	C is correct @
	C=C+CON	A,15
	C=C+CON	A,3
	B=B-1	A
	GOTO	dsADD
dsNOADD	D1=C

	LCHEX	#00018	one buffer "line"
	B=C	A
	LAHEX	#00005	loop (5+1) times
dsDOLINE	C=DAT1	3
	DAT0=C	3
	CD0EX
	C=C+B	A
	D0=C
	D1=D1+	3
	A=A-1	A
	GONC	dsDOLINE

dsNOBLOCK	D=D-1	A
	GOC	dsDOCOG
	GOTO	dsDOBLOCK
dsDOCOG	GOTO	dsDOCOLUMN


dsQUICK_DRAW	GOSUB	dsDRAW_SCN	draw bricks into RAM area
	GOSUBL	srRCL8_C	RAM address where bricks were drawn
	D1=C
	GOSUBL	srRCL9_A	ABUFF @
	LCHEX	#00045	offset to field
	C=C+A	A
	D0=C		screen pointer

	LAHEX	#0003B	(59+1) lines
dsQD_NEXT	C=DAT1	12	get first 12 nibbles
	DAT0=C	12	write them to screen
	D1=D1+	12	increment buffer pointer
	D0=D0+	12	increment screen pointer
	C=DAT1	12	get next 12 nibbles
	DAT0=C	12	write them to screen
	D1=D1+	12
	D0=D0+	12	|
	D0=D0+	10	| next line
	A=A-1	A
	GONC	dsQD_NEXT

	RTN		go home


sdDISOLVE	GOSUB	dsDRAW_SCN	draw bricks into RAM area
	GOSUBL	cfCLEAR	clear the playing field
	GOSUBL	srRCL8_C	stack GROB @
	R1=C.F	A	top-left
	GOSUBL	srRCL9_C	PICT GROB @
	LAHEX	#00045	offset to field
	C=C+A	A
	R2=C.F	A	store it here

	LCHEX	#00500	a little less than #5A0
	R3=C.F	A
sdNUTHER_ONE	A=0	W
	C=0	W
	D1=(5)	=TIMER2	TIMER2 addr
	A=DAT1	4	randomize the CRC
	D1=(5)	CRC	pseudo random number
	A=DAT1	4	get the 4 nibbles
	ASL	A	multiply by 16
	LCHEX	#005B2	#100000h / #2CFh
	GOSUBL	DIV	get a num in range [#0h,#2CFh]
	A=A+A	A	multiply by 2, even # in range [#0h, #59E]

	C=R1.F	A	get buffer @
	C=C+A	A	add random offset
	D1=C		put it here
	C=0	A	blank C
	C=DAT1	2	get 2 nibbles from buffer
	?C=0	A	is this zero?
	GOYES	sdNEXT	yes? get a different one
	LCHEX	#00018	nibbles per line in buffer
	GOSUBL	DIV	get line number
	LCHEX	#00022	nibbles per line in PICT
	GOSBVL	=MPY	get line offset
	C=C+B	A	add remainder (for column)
	A=R2.F	A	get PICT @
	C=C+A	A	add offset
	D0=C		put it here

	C=DAT1	2	read 2 nibbles (non-zero)
	DAT0=C	2	write to screen
	C=0	A	zero
	DAT1=C	2	now it's zero, we won't pick it again.
sdNEXT	C=R3.F	A
	C=C-1	A
	R3=C.F	A
	?C=0	A
	GOYES	sdNEXT2
	GOTO	sdNUTHER_ONE	do another nibble

sdNEXT2	LCHEX	#0003B	59+1 times
	D=C	A
	A=R1.F	A	buffer @
	D1=A		put it here to start
	A=R2.F	A	PICT @
	D0=A		put it here to start

sdPARSE	LCHEX	#00017	23+1 nibbles per line in buffer
	B=C	A
	C=0	A	make sure C is zero
sdREAD	C=DAT1	1	get 1 nibble from buffer
	?C=0	A	is it zero?
	GOYES	sdPARSE_NEXT	yes? This one is already done
	DAT0=C	1	otherwise, write it.
sdPARSE_NEXT	D1=D1+	1	next nibble in buffer
	D0=D0+	1	next nibble in PICT
	B=B-1	A
	GONC	sdREAD	do next nibble
	D0=D0+	10	move to next PICT line
	D=D-1	A
	GONC	sdPARSE	do another line

	RTN		RTN from sdDISOLVE


* isINIT_SCN will draw the entire screen,
* leaving the playing field untouched.

isINIT_SCN	GOSUBL	srRCL9_A	ABUFF bitmap
	D0=A
	LCHEX	8
	DAT0=C	1
	D0=D0+	1
	LCHEX	#FFFFFFFFFFFF
	DAT0=C	12
	D0=D0+	12
	DAT0=C	12
	D0=D0+	12
	D0=D0+	9	now pointing to next line

	LCHEX	#C
	DAT0=C	1
	D0=D0+	1
	LCHEX	#F	the others are still FFFF...
	DAT0=C	12
	D0=D0+	12
	DAT0=C	12
	D0=D0+	12
	D0=D0+	9	now pointing to next line

	LAHEX	0003B	(60-1) lines
	LCHEX	#C
isLEFT_SIDE	DAT0=C	1
	D0=D0+	16	|
	D0=D0+	16	|
	D0=D0+	2	| move to next line
	A=A-1	A
	GONC	isLEFT_SIDE

	DAT0=C	1
	D0=D0+	1
	LCHEX	#F	the others are still FFFF...
	DAT0=C	12
	D0=D0+	12
	DAT0=C	12
	D0=D0+	12
	D0=D0+	9	now pointing to next line

	LCHEX	#8
	DAT0=C	1
	D0=D0+	1
	LCHEX	#F	the others are still FFF...
	DAT0=C	12
	D0=D0+	12
	DAT0=C	12

	C=PC
	GOTO	isNEXT

	NIBHEX	1000000030000000
	NIBHEX	36C813603925A490
	NIBHEX	3925A4903925A490
	NIBHEX	3925A49036C81360
	NIBHEX	3000000030000000
	NIBHEX	BFFFFFF730000000
	NIBHEX	B0000C81B06A0252
	NIBHEX	B09A2252B0FA0252
	NIBHEX	B01A2252B7E40C81
	NIBHEX	30000000BFFFFFF7
	NIBHEX	3000000037000000
	NIBHEX	39CC94E03725A410
	NIBHEX	3925A4603925A480
	NIBHEX	37C4277030000000
	NIBHEX	3FFFFFF3BFFFFFF7
	NIBHEX	BFFFFFF73FFFFFF3
	NIBHEX	30000000BFFFFFF7
	NIBHEX	3000800030008000
	NIBHEX	3EF7800031088000
	NIBHEX	3108800031088000
	NIBHEX	310880003EF78000
	NIBHEX	3000800030008000
	NIBHEX	BFFFFFF730000000

* These are for the beta version.  This displays a little beta
* sign in the lower right-hand side.
*
*	NIBHEX	B9B39032BA09A055
*	NIBHEX	B919B055BA092055
*	NIBHEX	B939A03230000000
*	NIBHEX	B4274235B5522555
*	NIBHEX	B6522532B4224212

* These are for the real game. This is the little diamonds graphic
* in the lower right-hand side.
*	NIBHEX	BAAAAAA231545545
*	NIBHEX	B0282AA230005155
*	NIBHEX	B028A02231544045
*	NIBHEX	BA8280A235054155
*	NIBHEX	BA8AAA8235555555

* This is the new 2.0 graphic

	NIBHEX	BA0AAAA231155555
	NIBHEX	B0A8AAA231501454
	NIBHEX	BAA8882035455411
	NIBHEX	B28A2AA231051511
	NIBHEX	B28A8A2035441444

	NIBHEX	3000000030000000
	NIBHEX	3E0783C13F9FC7E3
	NIBHEX	3F9FC7E33F9FC7E3
	NIBHEX	3E0783C110000000

isNEXT	C=C+CON	A,4
	D1=C		address of bitmap
	GOSUBL	srRCL9_C	get ABUFF bitmap again
	LAHEX	#00019	offset to right side
	C=C+A	A
	D0=C
	LAHEX	#0003F	(64-1) lines
isRIGHT_SIDE	C=DAT1	8	read 8 nibbles of picture
	DAT0=C	8	write them to screen
	D1=D1+	8	increment to next part
	D0=D0+	16	|
	D0=D0+	16	|
	D0=D0+	2	| move D0 to next line
	A=A-1	A	decrement counter
	GONC	isRIGHT_SIDE

	RTN		screen is done!


* dmDRAW_MENU draws the menu options into the playing field.
* It assumes that the field is already clear, and it does not draw any
* portion of the Diamonds logo.
* It will check for the current speed toggle var (sr41) and display
* (SLOW, NORMAL, FAST, or SUPER) depending on that var.

dmDRAW_MENU	C=PC
	GOTO	dmNEXT

	NIBHEX	0000F1FC9C3087C37A397E40
	NIBHEX	E4105A259A4049A45A259A40
	NIBHEX	25505A259A0041A4DA659A50
	NIBHEX	E8005A259A6041A75BA59A60
	NIBHEX	29405A259A4041A45A259A40
	NIBHEX	E8005A259A4049A45A259A40
	NIBHEX	0000F1F87C3087E47A397E40
	NIBHEX	000000000000000000000000
	NIBHEX	EFFFFFFFFFFFFFFFFFFFFFF7
	NIBHEX	000000000000000000000000
	NIBHEX	6660831C42C93629947C1000
	NIBHEX	29408412524219E5A5920000
	NIBHEX	2F40831E51421FE5A69C0000
	NIBHEX	2940801290421925A4901000
	NIBHEX	696080F290C93929947E0000
	NIBHEX	000000000000000000000000
	NIBHEX	6760847C528336EC37000000
	NIBHEX	294084224248492580000000
	NIBHEX	2740872AD38909EC13000000
	NIBHEX	29408422520A492504000000
	NIBHEX	6760847C52C1362DB3000000
	NIBHEX	000000000000000000000000
	NIBHEX	6660837C8937CC3000000000
	NIBHEX	294084925212250000000000
	NIBHEX	2140837E50122C1000000000
	NIBHEX	294080925212250000000000
	NIBHEX	666080929117CC3000000000
	NIBHEX	000000000000000000000000

* This here only displays the [D], then blank.
	NIBHEX	676000000000000000000000
	NIBHEX	294000000000000000000000
	NIBHEX	294000000000000000000000
	NIBHEX	294000000000000000000000
	NIBHEX	676000000000000000000000

	NIBHEX	000000000000000000000000

* This displays [E] HELP
	NIBHEX	6F6084F2C100000000000000
	NIBHEX	214084124200000000000000
	NIBHEX	27408772C100000000000000
	NIBHEX	214084124000000000000000
	NIBHEX	6F6084FE5000000000000000

dmNEXT	C=C+CON	A,4
	D1=C		address of menu options GROB
	GOSUBL	srRCL9_C	get ABUFF bitmap address
	LAHEX	#002ED	offset to {#4h #16h}
	C=C+A	A
	D0=C

	LAHEX	#00026	(39-1) lines
dmNEXT_LINE	C=DAT1	12	read 12 nibbles of graphic
	DAT0=C	12	write 12 nibbles
	D1=D1+	12	increment GROB pointer
	D0=D0+	12	increment screen pointer
	C=DAT1	12	read 12 more nibbles
	DAT0=C	12	write 12 more
	D1=D1+	12	increment GROB pointer
	D0=D0+	12	|
	D0=D0+	10	| point to next line
	A=A-1	A
	GONC	dmNEXT_LINE	do the next line


	C=PC
	GOTO	dmDRAW_SPEED

	NIBHEX	071C428B3FED100	SLOW SPEED
	NIBHEX	801252484124200
	NIBHEX	0312D38937E4200
	NIBHEX	0412D30A0124200
	NIBHEX	83FC42C90FED100

	NIBHEX	846E42310EECB77	NORMAL SPEED
	NIBHEX	8592DB410125809
	NIBHEX	869ECB7106EC939
	NIBHEX	84925A410824809
	NIBHEX	84625A4F072CB77

	NIBHEX	876CD1CD97FE000	FAST SPEED
	NIBHEX	80928024A012100
	NIBHEX	83FC80CC9372100
	NIBHEX	809090058012100
	NIBHEX	809E80E487FE000

	NIBHEX	079ECB3077EDB30	SUPER SPEED
	NIBHEX	809258480924840
	NIBHEX	039EC93037EC940
	NIBHEX	049248404124840
	NIBHEX	8362CB4831EDB30

dmDRAW_SPEED	C=C+CON	A,4
	B=C	A	address of SLOW

	GOSUBL	srRCL41_C	get speed-toggle var
	LA(5)	SLOW_SPD
	?A=C	A	are we in SLOW speed?
	GOYES	dmGOT_SPEED

	LAHEX	#0004B
	B=B+A	A	increment to NORMAL speed
	LA(5)	NORMAL_SPD
	?A=C	A
	GOYES	dmGOT_SPEED

	LAHEX	#0004B
	B=B+A	A	increment to FAST speed
	LA(5)	FAST_SPD
	?A=C	A
	GOYES	dmGOT_SPEED

	LAHEX	#0004B
	B=B+A	A	increment to SUPER speed

dmGOT_SPEED	C=B	A	get address
	D1=C	A	address of proper speed
	GOSUBL	srRCL9_C	get ABUFF bitmap address
	LAHEX	#006A9	offset to {#14h #32h}
	C=C+A	A
	D0=C

	LAHEX	#00004	(4+1) lines
dmSPEED_LINE	C=DAT1	15	read one line of graphic
	DAT0=C	15	write 12 nibbles
	D1=D1+	15	increment GROB pointer
	D0=D0+	12	|
	D0=D0+	12	| increment screen pointer
	D0=D0+	10	| to next line

	A=A-1	A
	GONC	dmSPEED_LINE	do the next line

	RTN		done with dmDRAW_MENU


* dmDRAW_HLP draws the help screen that can be reached from the
* main menu.  It assumes that the field is already clear.
* Added 7/14/1995.  This uses a different method of finding the internal
* bitmap data.  (Using a gosub and C=RSTK).  I recommend this method
* above the others used in Diamonds.  I know I could go and fix all the
* coding errors in Diamonds, but who has time?  

dmDRAW_HLP	GOSUB	dmDHNEXT	Go to the drawing portion.  A gosub will
			Push the address of the data onto RSTK.

	NIBHEX	00C7EC374E93D7C30870E100
	NIBHEX	0049AA4D65A259A404A05200
	NIBHEX	0049AA4555A659A000605200
	NIBHEX	0049AA7555AA59C308105200
	NIBHEX	0049AA4545A2590504105200
	NIBHEX	0049AA4545A2592504165200
	NIBHEX	00C7EE474E93D7C30CF6E100
	NIBHEX	000000000000000000000000
	NIBHEX	00CFFFFFFFFFFFFFFFFFF300
	NIBHEX	000000000000000000000000
	NIBHEX	00400000E000005200000000
	NIBHEX	004000001100001200000000
	NIBHEX	0076293759969C5FC0EC4900
	NIBHEX	084925A0DDB19252E112D610
	NIBHEX	084925A01480925220125210
	NIBHEX	0076C937E8B0EC5CC5EC4210
	NIBHEX	000000200000000000000000
	NIBHEX	000008100000000000000000
	NIBHEX	000000000000000000000000
	NIBHEX	000000000000000000000000
	NIBHEX	06664373487FE30000000000
	NIBHEX	029449024801800000000000
	NIBHEX	02F429624837800000000000
	NIBHEX	029419424801800000000000
	NIBHEX	06961373CB71800000000000
	NIBHEX	000000000000000000000000
	NIBHEX	06F64B03C9379E3000000000
	NIBHEX	021449024290980000000000
	NIBHEX	02742902C196F80000000000
	NIBHEX	021419024294980000000000
	NIBHEX	06161B734A37980000000000
	NIBHEX	000000000000000000000000
	NIBHEX	00063713C97728924871DB00
	NIBHEX	000251124A0924A248015800
	NIBHEX	00025312C9372C31483AC900
	NIBHEX	000251124A012421480A4800
	NIBHEX	000637734A71E521CB74CB70
	NIBHEX	000000000000000000000000
	NIBHEX	0000601383392D189446ED30
	NIBHEX	0000211248496524A5492400
	NIBHEX	0000ABE28949A524A629EC10
	NIBHEX	000025020A492524A4192400
	NIBHEX	00006403C1362D1894162400
	NIBHEX	000000000000000000000000
	NIBHEX	0BB47733C139CD3000000000
	NIBHEX	098521524A49240000000000
	NIBHEX	09962332C979CC1000000000
	NIBHEX	098421524849050000000000
	NIBHEX	0BB427534846EC3000000000
	NIBHEX	000000000000000000000000
	NIBHEX	000003238947F10000000000
	NIBHEX	000001324A42400000000000
	NIBHEX	000009F24A42400000000000
	NIBHEX	000001324942400000000000
	NIBHEX	000003238237400000000000

dmDHNEXT	C=RSTK		Get the address of the bitmap into C[A]
	D1=C
	GOSUBL	srRCL9_C	get ABUFF bitmap address
	LA(5)	103	offset to {#4h #3h}
	C=C+A	A
	D0=C

	LA(2)	(55-1)	(55-1) lines
dmDHLOOP	C=DAT1	12	read 12 nibbles of graphic
	DAT0=C	12	write 12 nibbles
	D1=D1+	12	increment GROB pointer
	D0=D0+	12	increment screen pointer
	C=DAT1	12	read 12 more nibbles
	DAT0=C	12	write 12 more
	D1=D1+	12	increment GROB pointer
	D0=D0+	12	|
	D0=D0+	10	| point to next line
	A=A-1	B
	GONC	dmDHLOOP	do the next line

dmDH_KEY	LCHEX	#1FF	| wait here until
	OUT=C		| they stop pressing
	GOSBVL	=CINRTN	| a key
	LAHEX	#0003F	|
	A=A&C	A	|
	?A#0	A	|
	GOYES	dmDH_KEY	|

	LCHEX	#00800	about 10 seconds
	GOSUBL	waWAIT
	GOLONG	mnMENU	Return to Menu


* cfPUT_LOGO will clear the playing field and put the Diamonds
* Logo at the top (top 18 lines).  cfPUT_LOGO uses cfCLEAR.

cfPUT_LOGO	GOSUBL	srRCL9_A	PICT GROB data @
	LCHEX	#00045	offset to field
	C=C+A	A
	D0=C
	GOSUBL	srRCL8_A
	LCHEX	#005B0	offset to logo
	C=C+A	A
	D1=C

	LCHEX	#00011	17+1 lines
cfPUT_LINE	A=DAT1	12	get 12 nibbles from storage
	DAT0=A	12	put them on screen
	D0=D0+	12	inc 12 nibbles
	D1=D1+	12	ditto
	A=DAT1	12	get 12 more
	DAT0=A	12	write 12 more
	D0=D0+	12	you know.
	D1=D1+	12	ditto
	D0=D0+	10	move to next PICT line

	C=C-1	A
	GONC	cfPUT_LINE	do next line

	LCHEX	#00029	41+1 lines left
	GOTO	cfCHEAT	finish clearing the rest.

* cfCLEAR will clear the entire playing field

cfCLEAR	GOSUBL	srRCL9_A	PICT GROB data @
	LCHEX	#00045	offset to field
	C=C+A	A
	D0=C
	LCHEX	#0003B	59+1 lines
cfCHEAT	A=0	W
cfERASE_LINE	DAT0=A	12	blank 12 nibbles
	D0=D0+	12	inc 12 nibbles
	DAT0=A	12	blank 12 more
	D0=D0+	12	you know.
	D0=D0+	10	move to next line
	C=C-1	A
	GONC	cfERASE_LINE	do next line
	RTN


* --------------------------------------------------------
* This is a number input routine (and it uses fdFONT)
* The active keys are:
* [0]-[9]  the numbers
* [ENTER]  accept
* [<-]     backspace
* [ON]     Exit to RPL
*
* all other keys are ignored
*
* On entry, C should contain a pixel # to display numbers
* on screen (same format as fdFONT).
* On exit, C == the number entered (0 if none entered).
* --------------------------------------------------------

inINPUT_NUM	GOSUBL	srSTO31_C	store pixel # (coords)
	C=0	A
	GOSUBL	srSTO34_C
	GOSUBL	srSTO35_C

inCHECK_KEY	LCHEX	#1FF	| wait here until
	OUT=C		| they stop pressing
	GOSBVL	=CINRTN	| a key
	LAHEX	#0003F	|
	A=A&C	A	|
	?A#0	A	|
	GOYES	inCHECK_KEY	|

inGOOD_KEY	LCHEX	#01F	bottom 5 rows of keys
	OUT=C
	GOSBVL	=CINRTN
	LAHEX	#0001F	keys we want (and some we don't...)
	A=A&C	A
	?A#0	A
	GOYES	inHIT_KEY

	GOSUBL	srRCL31_C	get coords
	GOSUBL	cuCURSOR	draw the cursor

	LCHEX	#400	(CLK fix)
	OUT=C
	GOSBVL	=CINRTN
	LAHEX	#08000	[ON]
	A=A&C	A
	?A=0	A	no press?
	GOYES	inGOOD_KEY
	GOLONG	maEND

inHIT_KEY	GOSUB	inNUMS
	D0=C		D0 pointing to "0"

	LCHEX	#010	[ENTER] or [<-]
	OUT=C
	GOSBVL	=CINRTN
	LAHEX	#00011
	A=A&C	A
	?A=0	A
	GOYES	in789
	GOTO	inROW4

in789	LCHEX	#008	[7] [8] [9]
	OUT=C
	GOSBVL	=CINRTN
	LAHEX	#0000E
	A=A&C	A
	?A=0	A
	GOYES	in456
	GOTO	inROW3

in456	LCHEX	#004	[4] [5] [6]
	OUT=C
	GOSBVL	=CINRTN
	LAHEX	#0000E
	A=A&C	A
	?A=0	A
	GOYES	in123
	GOTO	inROW2

in123	LCHEX	#002	[1] [2] [3]
	OUT=C
	GOSBVL	=CINRTN
	LAHEX	#0000E
	A=A&C	A
	?A=0	A
	GOYES	in0
	GOTO	inROW1

in0	LCHEX	#001	[0]
	OUT=C
	GOSBVL	=CINRTN
	LAHEX	#00008
	A=A&C	A
	?A#0	A
	GOYES	inROW0
	GOTO	inCHECK_KEY

inROW0	B=0	A	the # is 0

inCHECK_OK	GOSUBL	srRCL34_C	number of keys pressed so far
	LAHEX	#00002	2 is the max
	?A#C	A
	GOYES	inOK
	GOTO	inERROR	make a blip

inOK	C=C+1	A	one more key pressed
	GOSUBL	srSTO34_C
	GOSUBL	srRCL35_C	get the #
	CSL	A	shift left 1 nibble
	C=C+B	A	add new nibble on
	GOSUBL	srSTO35_C	store it

	GOSUBL	srRCL31_C	get coords
	GOSUB	cuCURSOR_OFF
	GOSUBL	fdFONT	draw the number
	GOSUBL	srSTO31_C	store coords.

	GOTO	inCHECK_KEY

inROW1	LCHEX	#00001
	B=C	A	a 1
	D0=D0+	4	D0 points to "1"

inCOLUMN	?ABIT=1	3	check bit 3
	GOYES	inCHECK_OK
	B=B+1	A	next num
	D0=D0+	4	point D0 to next num

	?ABIT=1	2	check bit 2
	GOYES	inCHECK_OK
	B=B+1	A	next num
	D0=D0+	4	point D0 to next num
	GOTO	inCHECK_OK	we (correctly) assume it was bit 1

inROW2	LCHEX	#00004	a 4
	B=C	A
	D0=D0+	16	D0 points to "4"
	GOTO	inCOLUMN

inROW3	LCHEX	#00007	a 7
	B=C	A
	D0=D0+	16
	D0=D0+	12	D0 points to "7"
	GOTO	inCOLUMN

inROW4	LAHEX	#00001	[BACKSPACE]
	A=A&C	A
	?A#0	A	was it a backspace?
	GOYES	inBACKSPACE

	LAHEX	#00010	[ENTER]
	A=A&C	A
	?A#0	A	was it an enter?
	GOYES	inENTER
	GOTO	inCHECK_KEY

inENTER	GOSUBL	srRCL31_C	coords
	GOSUB	cuCURSOR_OFF
	GOSUBL	srRCL35_C	get the # (it's 0 if you didn't type)
	GOLONG	mnIN_RTN	instead of RTN

inBACKSPACE	GOSUBL	srRCL34_C	get # of digits so far
	?C#0	A	have you typed some?
	GOYES	inBS_OK
	GOTO	inERROR	make a blip

inBS_OK	C=C-1	A	one less num now...
	GOSUBL	srSTO34_C	save it
	GOSUBL	srRCL35_C	the #
	CSR	A	lose a nibble
	GOSUBL	srSTO35_C	save it

	GOSUBL	srRCL31_C	coords
	GOSUB	cuCURSOR_OFF
	C=C-CON	A,5	back up 5 pixels
	GOSUBL	srSTO31_C	save it
	GOSUB	ccCONVERT	return nibble offset in C, shifts in B

	GOSUBL	srRCL9_A	PICT @
	A=A+C	A	add nibble offset
	D0=A		put it here

	LAHEX	#000F0	mask
inSHIFT	?B=0	A	do some?
	GOYES	inNOSHIFT	no? skip
	A=A+A	A	shift left 1 bit (right on screen)
	A=A+1	A	make bit #0 a 1
	B=B-1	A	one less shift
	GOTO	inSHIFT

inNOSHIFT	LCHEX	#00005	(5+1) times
	D=C	A

inERASE	C=DAT0	2	get 2 screen nibbles
	C=C&A	A	mask off old num
	DAT0=C	2	write it back to screen

	LCHEX	#00022
	B=C	A
	CD0EX		get D0
	C=C+B	A
	D0=C

	D=D-1	A
	GONC	inERASE	do another line

	GOTO	inCHECK_KEY

inERROR	?ST=1	11	sound off?
	GOYES	inE_END

	LCHEX	#00008	note length
	D=C	A

inE_AGAIN	LCHEX	#800
	OUT=C
	LCHEX	#00080	pitch #1
inE_DEC1	C=C-1	A
	GONC	inE_DEC1
	C=0	A
	OUT=C
	LCHEX	#00080	pitch #2
inE_DEC2	C=C-1	A
	GONC	inE_DEC2

	D=D-1	A
	GONC	inE_AGAIN

inE_END	GOTO	inCHECK_KEY


* This routine will return C pointing to the "0"

inNUMS	C=PC
	GOTO	inNEXT

	NIBHEX	030013002300
	NIBHEX	330043005300
	NIBHEX	630073008300
	NIBHEX	9300

inNEXT	C=C+CON	A,4
	RTN



* --------------------------------------------------------
* This is a letter input routine (and it uses fdFONT)
* The active keys are:
* [A]-[F]  the letters
* [ENTER]  accept
* [<-]     backspace
* [ON]     Exit to RPL
*
* all other keys are ignored
*
* On entry, C should contain a pixel # to display numbers
* on screen (same format as fdFONT).
* On exit, C == the number entered (0 if none entered).
*
* the whole R2 register is used to keep track of the
* letters pressed.
* --------------------------------------------------------

ilINPUT_LET	GOSUBL	srSTO31_C	store pixel # (coords)
	C=0	W
	GOSUBL	srSTO34_C
	R2=C		the whole reg (save letters here)

	D0=(5)	#0010B	annunciators
	C=DAT0	1	get 1 nibble
	CBIT=1	2	alpha
	DAT0=C	1	turn on alpha annunciator

ilCHECK_KEY	LCHEX	#1FF	| wait here until
	OUT=C		| they stop pressing
	GOSBVL	=CINRTN	| a key
	LAHEX	#0003F	|
	A=A&C	A	|
	?A#0	A	|
	GOYES	ilCHECK_KEY	|

ilGOOD_KEY	LCHEX	#1FF	top 5 rows of keys (and then some)
	OUT=C
	GOSBVL	=CINRTN
	LAHEX	#0001F	keys we want (and some we don't...)
	A=A&C	A
	?A#0	A
	GOYES	ilHIT_KEY

	GOSUBL	srRCL31_C	get coords
	GOSUB	cuCURSOR	do the cursor

	LCHEX	#400	(CLK fix)
	OUT=C
	GOSBVL	=CINRTN
	LAHEX	#08000	[ON]
	A=A&C	A
	?A=0	A	no press?
	GOYES	ilGOOD_KEY
	GOLONG	maEND

ilHIT_KEY	C=PC
	GOTO	ilNEXT

	NIBHEX	140024003400	row 1
	NIBHEX	440054006400	[A B C D E F]

	NIBHEX	740084009400	row 2
	NIBHEX	A400B400C400	[G H I J K L]

	NIBHEX	D400E400F400	row 3
	NIBHEX	050015002500	[M N O P Q R]

	NIBHEX	350045005500	row 4
	NIBHEX	650075008500	[S T U V W X]

	NIBHEX	9500A500	row 5  [Y Z]

ilNEXT	C=C+CON	A,4
	D0=C		D0 pointing to "A"

	LCHEX	#010	[ENTER], [<-] Y or Z
	OUT=C
	GOSBVL	=CINRTN
	LAHEX	#0001D
	A=A&C	A
	?A=0	A
	GOYES	ilBCDEF
	GOTO	ilROW5

ilBCDEF	LCHEX	#100	B C D E F
	OUT=C
	GOSBVL	=CINRTN
	LAHEX	#0001F
	A=A&C	A
	?A=0	A
	GOYES	ilA
	GOTO	ilROW1

ilA	LCHEX	#002	A
	OUT=C
	GOSBVL	=CINRTN
	LAHEX	#00010
	A=A&C	A
	?A=0	A
	GOYES	ilHIJKL
	GOTO	ilSP_A	'cause A is special

ilHIJKL	LCHEX	#080	H I J K L
	OUT=C
	GOSBVL	=CINRTN
	LAHEX	#0001F
	A=A&C	A
	?A=0	A
	GOYES	ilG
	GOTO	ilROW2

ilG	LCHEX	#004	G
	OUT=C
	GOSBVL	=CINRTN
	LAHEX	#00010
	A=A&C	A
	?A=0	A
	GOYES	ilNOPQR
	GOTO	ilSP_G	'cause G is special

ilNOPQR	LCHEX	#040	N O P Q R
	OUT=C
	GOSBVL	=CINRTN
	LAHEX	#0001F
	A=A&C	A
	?A=0	A
	GOYES	ilM
	GOTO	ilROW3

ilM	LCHEX	#001	M
	OUT=C
	GOSBVL	=CINRTN
	LAHEX	#00010
	A=A&C	A
	?A=0	A
	GOYES	ilTUVWX
	GOTO	ilSP_M	'cause M is special

ilTUVWX	LCHEX	#020	T U V W X
	OUT=C
	GOSBVL	=CINRTN
	LAHEX	#0001F
	A=A&C	A
	?A=0	A
	GOYES	ilS
	GOTO	ilROW4

ilS	LCHEX	#008	S
	OUT=C
	GOSBVL	=CINRTN
	LAHEX	#00010
	A=A&C	A
	?A=0	A
	GOYES	ilGO_AWAY
	GOTO	ilSP_S	'cause S is special

ilGO_AWAY	GOTO	ilCHECK_KEY

ilSP_A	LCHEX	#00041
	B=C	A	an "A"
	GOTO	ilCHECK_OK	DO is already pointing to "A"

ilROW1	LCHEX	#00042
	B=C	A	a "B"
	D0=D0+	4	D0 points to "B"
	GOTO	ilCOLUMN

ilSP_G	LCHEX	#00018	offset to "G"
	B=C	A
	CD0EX
	C=C+B	A
	D0=C		D0 points to "G"
	LCHEX	#00047
	B=C	A	a "G"
	GOTO	ilCHECK_OK

ilROW2	LCHEX	#0001C	offset to "H"
	B=C	A
	CD0EX
	C=C+B	A
	D0=C		D0 points to "H"
	LCHEX	#00048	an "H"
	B=C	A
	GOTO	ilCOLUMN

ilSP_M	LCHEX	#00030	offset to "M"
	B=C	A
	CD0EX
	C=C+B	A
	D0=C		D0 points to "M"
	LCHEX	#0004D
	B=C	A	an "M"
	GOTO	ilCHECK_OK

ilROW3	LCHEX	#00034	offset to "N"
	B=C	A
	CD0EX
	C=C+B	A
	D0=C		D0 points to "N"
	LCHEX	#0004E	an "N"
	B=C	A
	GOTO	ilCOLUMN

ilSP_S	LCHEX	#00048	offset to "S"
	B=C	A
	CD0EX
	C=C+B	A
	D0=C		D0 points to "S"
	LCHEX	#00053	an "S"
	B=C	A
	GOTO	ilCHECK_OK

ilROW4	LCHEX	#0004C	offset to "T"
	B=C	A
	CD0EX
	C=C+B	A
	D0=C		D0 points to "T"
	LCHEX	#00054	a "T"
	B=C	A

ilCOLUMN	LCHEX	#00003	3+1 times
	D=C	A
ilC_NEXT	?ABIT=1	4	check bit
	GOYES	ilCHECK_OK
	B=B+1	A	next letter
	A=A+A	A	shift bits left
	D0=D0+	4	point D0 to next letter
	D=D-1	A
	GONC	ilC_NEXT	if we drop out, assume it's bit 1

ilCHECK_OK	GOSUBL	srRCL34_C	number of keys pressed so far
	LAHEX	#00003	3 is the max
	?A#C	A
	GOYES	ilOK
	GOTO	ilERROR	make a blip

ilOK	C=C+1	A	one more key pressed
	GOSUBL	srSTO34_C
	C=R2		get the letters
	CSL	W	shift left 1 nibble
	CSL	W	shift left another nibble
	C=C+B	A	add new nibbles on
	R2=C		store it

	GOSUBL	srRCL31_C	get coords
	GOSUB	cuCURSOR_OFF	turn off cursor
	GOSUB	fdFONT	draw the letter
	GOSUBL	srSTO31_C	store coords.

	GOTO	ilCHECK_KEY

ilROW5	LAHEX	#00001	[BACKSPACE]
	A=A&C	A
	?A=0	A	was it a backspace?
	GOYES	ilC_ENTER	no? check ENTER
	GOTO	ilBACKSPACE

ilC_ENTER	LAHEX	#00010	[ENTER]
	A=A&C	A
	?A#0	A	was it an enter?
	GOYES	ilENTER

	LAHEX	#00008	Y
	A=A&C	A
	?A=0	A	was it Y?
	GOYES	ilR4_Z	no? check for Z
	LCHEX	#00059	a "Y"
	B=C	A
	LAHEX	#00060	offset to Y
ilR4CHEAT	CD0EX
	C=C+A	A
	D0=C
	GOTO	ilCHECK_OK

ilR4_Z	LAHEX	#00004	Z
	A=A&C	A
	?A=0	A	was it Z?
	GOYES	ilR4_BAD	nope, outta here!
	LCHEX	#0005A	a "Z"
	B=C	A
	LAHEX	#00064	offset to Z
	GOTO	ilR4CHEAT

ilR4_BAD	GOTO	ilCHECK_KEY

ilENTER	D0=(5)	#0010B	annunciators
	C=DAT0	1	get 1 nibble
	CBIT=0	2	alpha
	DAT0=C	1	turn off alpha annunciator

	GOSUBL	srRCL31_C	coords
	GOSUB	cuCURSOR_OFF

	C=R2		get the letters (0 if you didn't type)
	GOTO	dhIL_RTN	instead of RTN

ilBACKSPACE	GOSUBL	srRCL34_C	get # of letters so far
	?C#0	A	have you typed some?
	GOYES	ilBS_OK
	GOTO	ilERROR	make a blip

ilBS_OK	C=C-1	A	one less letter now...
	GOSUBL	srSTO34_C	save it
	C=R2		the letters
	CSR	W	lose a nibble
	CSR	W	lose another nibble
	R2=C		save it

	GOSUBL	srRCL31_C	coords
	GOSUB	cuCURSOR_OFF
	C=C-CON	A,5	back up 5 pixels
	GOSUBL	srSTO31_C	save it
	GOSUB	ccCONVERT	return nibble offset in C, shifts in B

	GOSUBL	srRCL9_A	PICT @
	A=A+C	A	add nibble offset
	D0=A		put it here

	LAHEX	#000F0	mask
ilSHIFT	?B=0	A	do some?
	GOYES	ilNOSHIFT	no? skip
	A=A+A	A	shift left 1 bit (right on screen)
	A=A+1	A	make bit #0 a 1
	B=B-1	A	one less shift
	GOTO	ilSHIFT

ilNOSHIFT	LCHEX	#00005	(5+1) times
	D=C	A

ilERASE	C=DAT0	2	get 2 screen nibbles
	C=C&A	A	mask off old num
	DAT0=C	2	write it back to screen

	LCHEX	#00022
	B=C	A
	CD0EX		get D0
	C=C+B	A
	D0=C

	D=D-1	A
	GONC	ilERASE	do another line

	GOTO	ilCHECK_KEY

ilERROR	?ST=1	11	sound off?
	GOYES	ilE_END

	LCHEX	#00008	note length
	D=C	A

ilE_AGAIN	LCHEX	#800
	OUT=C
	LCHEX	#00080	pitch #1
ilE_DEC1	C=C-1	A
	GONC	ilE_DEC1
	C=0	A
	OUT=C
	LCHEX	#00080	pitch #2
ilE_DEC2	C=C-1	A
	GONC	ilE_DEC2

	D=D-1	A
	GONC	ilE_AGAIN

ilE_END	GOTO	ilCHECK_KEY



* --------------------------------------------------------
* This routine converts a pixel number (in C) into
* a nibble offset (into PICT) in C, and the # of bit
* shifts in B.
*
* On Entry:
*   C is the pixel number (same format as for fdFONT)
*
* On Exit:
*   C=nibble offset into PICT
*   B=# of bit shifts needed
*   A	is used, and == C
* --------------------------------------------------------

ccCONVERT	A=0	W
	A=C	A
	C=0	W

	LCHEX	#00083	131 decimal
	GOSUBL	DIV
	C=B	A	remainder
	GOSUBL	srSTO36_C	save it for a sec.

	LCHEX	#00022	nibbles per line
	GOSBVL	=MUL#	multiply, res in B
	A=B	A

	GOSUBL	srRCL36_C	get back remainder
	B=0	A
	SB=0
	CSRB.F	A	divide by 2
	?SB=0		was there a bit lost?
	GOYES	ccNOBIT1	no?...continue
	B=B+1	A	need to shift once
ccNOBIT1	SB=0
	CSRB.F	A	divide by 2
	?SB=0		was there a bit lost?
	GOYES	ccNOBIT2	no?...continue
	B=B+CON	A,2	2 more shifts needed
ccNOBIT2	C=C+A	A	this is offset into PICT

	RTN


* --------------------------------------------------------
* This is a little cursor routine used by (in and il)
* C must contain pixel coords.
* R1 is used
* D0 is used
* A is used,
* B is used,
* C is unchanged
* --------------------------------------------------------

cuCURSOR	R1=C.F	A	save coords
	GOSUBL	srRCL39_C	counter
	C=C-1	A
	GOSUBL	srSTO39_C
	?C=0	A
	GOYES	cuDOIT
	C=R1.F	A	get coords back
	RTN

cuDOIT	LCHEX	#00050	new counter
	GOSUBL	srSTO39_C
	C=R1.F	A	get coords
	GOSUB	cuCURSOR_OFF
	GOSUBL	srRCL40_C	get cursor offset
	LAHEX	#00022	down 1 line
	C=C+A	A
	LAHEX	#000CC	too far
	?C#A	A
	GOYES	cuOK
	C=0	A
cuOK	GOSUBL	srSTO40_C

	C=R1.F	A	get coords
	GOSUB	ccCONVERT
	GOSUBL	srRCL9_A	get PICT
	A=A+C	A	add coords offset
	GOSUBL	srRCL40_C	get cursor offset
	A=A+C	A
	D0=A		put it here

	LCHEX	#0F	the cursor
cuSHIFT	?B=0	A
	GOYES	cuNO_SHIFT
	C=C+C	A	shift left (right on screen)
	B=B-1	A
	GOTO	cuSHIFT

cuNO_SHIFT	A=DAT0	2	get 2 nibbles
	A=A!C	A	OR cursor on
	DAT0=A	2	put it on screen

	C=R1.F	A	get back coords
	RTN


* --------------------------------------------------------
* This will erase the cursor
*
* C must contain coords
* R1 is used
* R3 is used
* A is used
* B is used
* D0 is unchanged
* C is unchanged
* --------------------------------------------------------


cuCURSOR_OFF	R1=C.F	A	save coords for a sec
	CD0EX		get D0
	R3=C.F	A	put it here
	C=R1.F	A	get coords back
	GOSUB	ccCONVERT
	GOSUBL	srRCL9_A
	A=A+C	A	add coord offset
	GOSUBL	srRCL40_C	get cursor offset
	A=A+C	A
	D0=A

	LCHEX	#F0	the cursor (inverse)
cuSHIFT2	?B=0	A
	GOYES	cuNO_SHIFT2
	C=C+C	A	shift left (right on screen)
	CBIT=1	0	make bit 0 inversed
	B=B-1	A
	GOTO	cuSHIFT2

cuNO_SHIFT2	A=DAT0	2	get 2 nibbles from screen
	A=A&C	A	mask off cursor
	DAT0=A	2	fix it on screen

	C=R3.F	A	get D0
	D0=C		resore it
	C=R1.F	A	get C back
	RTN


* --------------------------------------------------------
* This is a font drawing routine.
* It will draw upper-case letters from (A-Z),
* numbers from (0-9), and a space.  Any other ASCII chars will
* give unprediced results (like garbage on the screen).
* All fonts are 4 pixels wide and 6 pixels tall.
*
* On Entry:
*   D0 points to a NULL terminated string.  As this string will be read
*     one byte at a time, the nibbles of each byte should be reversed.
*   C  contains the pixel number on the PICT bitmap.  PICT is considered
*      to be 131 pixels by 64 pixels.  So if you want to draw to {#50h #30h}
*      then C == 50 + 30 * 131 or, LCHEX #00F8C.  {#0h #0h} is valid.
*
* On Exit:
*   D0 points to one byte past the NULL character
*   C  points to 5 pixels past where the last character was drawn (and
*      where the next logical character would go).
* --------------------------------------------------------

fdFONT	GOSUBL	srSTO4_A	preserve A
	GOSUBL	srSTO7_C	preserve C
	C=B	A	get B
	GOSUBL	srSTO24_C	save it here
	C=D	A	get D
	GOSUBL	srSTO30_C	save it here
	CD1EX		get D1
	GOSUBL	srSTO27_C	save it here
	CD0EX
	D0=C		keep D0 intact
	GOSUBL	srSTO3_C	save string @ here

fdDO_ONE	LAHEX	#00041	ASCII "A"
	B=A	A
	A=DAT0	2	read 1 char
	?A#0	A	non NULL?
	GOYES	fdSPACE	check for space
	GOTO	fdEND	go to end
fdSPACE	LCHEX	#00020	a space
	?A#C	A	is this not a space?
	GOYES	fdCH_NUM	go check for num
	GOTO	fdNEXT_LET	treat it special
fdCH_NUM	?A<B	A	is it a number?
	GOYES	fdNUM

	ABEX	X
	B=B-A	A	convert to "A"==0, "B"==1, etc.
	A=B	A
	LCHEX	#00006	| multiply by six
	GOSBVL	=MUL#	| result in B
	GOSUB	laLETTERS	put @ of letters into A
	GOTO	fdOFFSET

fdNUM	LCHEX	#00030	ASCII "0"
	C=A-C	A	"0"==0, "1"==1, etc.
	LAHEX	#00006	|multiply by six
	GOSBVL	=MUL#	|result in B
	GOSUB	naNUMBERS	put @ of numbers into A

fdOFFSET	A=A+B	A	add offset to correct letter
	D1=A		D1 is set to letter address
	A=0	W	| prepare for DIV
	C=0	W	|
	GOSUBL	srRCL7_A	coordinates
	LCHEX	#00083	131 decimal
	GOSUBL	DIV
	C=B	A	remainder
	GOSUBL	srSTO29_C	save it for a sec.

	LCHEX	#00022	nibbles per line
	GOSBVL	=MUL#	multiply, res in B
	A=B	A

	GOSUBL	srRCL29_C	get back remainder
	B=0	A
	SB=0
	CSRB.F	A	divide by 2
	?SB=0		was there a bit lost?
	GOYES	fdNOBIT1	no?...continue
	B=B+1	A	need to shift once
fdNOBIT1	SB=0
	CSRB.F	A	divide by 2
	?SB=0		was there a bit lost?
	GOYES	fdNOBIT2	no?...continue
	B=B+CON	A,2	2 more shifts needed
fdNOBIT2	C=C+A	A	add lines, this is offset into PICT
	GOSUBL	srRCL9_A	get PICT address
	C=C+A	A
	D0=C		WOW! we figured it out!

	LCHEX	#00005	(5+1) lines per letter
	D=C	A
fdLINE	A=0	A
	A=DAT1	1	get 1 nibble of letter
	C=B	A	number of shifts
fdSHIFT	?C=0	A	do some?
	GOYES	fdNOSHIFT	no? continue
	A=A+A	A	shift left one bit (right on screen)
	C=C-1	A
	GOTO	fdSHIFT

fdNOSHIFT	C=DAT0	2	get 2 nibbles from screen
	C=C!A	A	or letter on.
	DAT0=C	2	write it back to screen.

	LCHEX	#00022
	AD0EX
	A=A+C	A	down 1 line on PICT
	D0=A
	D1=D1+	1	next letter nibble
	D=D-1	A
	GONC	fdLINE	do next line

fdNEXT_LET	GOSUBL	srRCL3_C	string address
	C=C+CON	A,2	increment 1 char
	GOSUBL	srSTO3_C	save it
	D0=C		put string @ here for next loop
	GOSUBL	srRCL7_C	get coordinates
	C=C+CON	A,5	increment 5 pixels
	GOSUBL	srSTO7_C	save it
	GOTO	fdDO_ONE

fdEND	D0=D0+	2	point D0 past NULL
	GOSUBL	srRCL27_A	get D1
	D1=A		restore it
	GOSUBL	srRCL24_A	get B
	B=A	A	restore it
	A=C	A
	GOSUBL	srRCL30_C	get D
	D=C	A	restore it
	GOSUBL	srRCL4_C	get A
	ACEX	A	switch
	RTN

ndNUMBER	GOSUBL	srSTO4_A
	GOSUBL	srSTO7_C
	C=B	A
	GOSUBL	srSTO24_C
	C=D	A
	GOSUBL	srSTO30_C
	CD1EX
	GOSUBL	srSTO27_C

	AD0EX
	C=A	A
	GOSUBL	srSTO31_C	preserve number here
	LCHEX	#00004
	D=C	A
	B=0	A
ndREVERSE	BSR	A
	LCHEX	#F0000
	C=C&A	A
	B=B!C	A
	ASL	A
	D=D-1	A
	GONC	ndREVERSE
	C=B	A	the number, reversed
	GOSUBL	srSTO3_C
	LAHEX	#00005	5 digits in a number
	R1=A.F	A

ndDO_ONE	LAHEX	#0000F	mask
	C=C&A	A	mask off nibble 0
	LAHEX	#00006
	GOSBVL	=MUL#
	GOSUB	naNUMBERS	get numbers @ in A

ndOFFSET	A=A+B	A	add offset to correct number
	D1=A		D1 is set to number @
	A=0	W	| prepare for DIV
	C=0	W	|
	GOSUBL	srRCL7_A	coordinates
	LCHEX	#00083	131 decimal
	GOSUBL	DIV
	C=B	A	remainder
	GOSUBL	srSTO29_C	save it for a sec.

	LCHEX	#00022	nibbles per line
	GOSBVL	=MUL#	multiply, res in B
	A=B	A

	GOSUBL	srRCL29_C	get back remainder
	B=0	A
	SB=0
	CSRB.F	A	divide by 2
	?SB=0		was there a bit lost?
	GOYES	ndNOBIT1	no?...continue
	B=B+1	A	need to shift once
ndNOBIT1	SB=0
	CSRB.F	A	divide by 2
	?SB=0		was there a bit lost?
	GOYES	ndNOBIT2	no?...continue
	B=B+CON	A,2	2 more shifts needed
ndNOBIT2	C=C+A	A	add lines, this is offset into PICT
	GOSUBL	srRCL9_A	get PICT address
	C=C+A	A
	D0=C		WOW! we figured it out!

	LCHEX	#00005	(5+1) lines per number
	D=C	A
ndLINE	A=0	A
	A=DAT1	1	get 1 nibble of number
	C=B	A	number of shifts
ndSHIFT	?C=0	A	do some?
	GOYES	ndNOSHIFT	no? continue
	A=A+A	A	shift left one bit (right on screen)
	C=C-1	A
	GOTO	ndSHIFT

ndNOSHIFT	C=DAT0	2	get 2 nibbles from screen
	C=C!A	A	OR number on.
	DAT0=C	2	write it back to screen.

	LCHEX	#00022
	AD0EX
	A=A+C	A	down 1 line on PICT
	D0=A
	D1=D1+	1	next number nibble
	D=D-1	A
	GONC	ndLINE	do next line

	GOSUBL	srRCL7_C	get coordinates
	C=C+CON	A,5	increment 5 pixels
	GOSUBL	srSTO7_C	save it
	GOSUBL	srRCL3_C	reversed number
	CSR	A	shift it right 1 nibble
	GOSUBL	srSTO3_C	save it
	A=R1.F	A
	A=A-1	A
	R1=A.F	A
	?A=0	A	done?
	GOYES	ndEND
	GOTO	ndDO_ONE

ndEND	GOSUBL	srRCL31_C
	D0=C		restore number
	GOSUBL	srRCL27_C	get D1
	D1=C		restore it
	GOSUBL	srRCL24_C	get B
	B=C	A	restore it
	GOSUBL	srRCL30_C	get D
	D=C	A	restore it
	GOSUBL	srRCL4_A	restore A
	GOSUBL	srRCL7_C	restore C
	RTN

laLETTERS	A=PC
	GOTO	laEND

	NIBHEX	699F99	A
	NIBHEX	797997	B
	NIBHEX	691196	C
	NIBHEX	799997	D
	NIBHEX	F1711F	E
	NIBHEX	F17111	F
	NIBHEX	691D9E	G
	NIBHEX	999F99	H
	NIBHEX	722227	I
	NIBHEX	888896	J
	NIBHEX	953599	K
	NIBHEX	11111F	L
	NIBHEX	9FF999	M
	NIBHEX	99BD99	N
	NIBHEX	699996	O
	NIBHEX	799711	P
	NIBHEX	69995A	Q
	NIBHEX	799799	R
	NIBHEX	E16887	S
	NIBHEX	722222	T
	NIBHEX	999996	U
	NIBHEX	999952	V
	NIBHEX	999FF9	W
	NIBHEX	999699	X
	NIBHEX	995222	Y
	NIBHEX	F8421F	Z

laEND	A=A+CON	A,4	A now pointing to @ of letters
	RTN

naNUMBERS	A=PC
	GOTO	naEND

	NIBHEX	699996	0
	NIBHEX	46444E	1
	NIBHEX	69861F	2
	NIBHEX	F84896	3
	NIBHEX	999F88	4
	NIBHEX	F17896	5
	NIBHEX	E17996	6
	NIBHEX	F84222	7
	NIBHEX	696996	8
	NIBHEX	699E87	9

naEND	A=A+CON	A,4	A is now pointing to @ of numbers
	RTN


* slSAVE_LOGO will get the Diamonds logo from PICT (top 18 lines
* of the playing field, 24 nibbles * 18 lines == #1B0h nibbles.
* and save it to sr8 + #5B0h.  Since sd uses #5A0 nibbles here,
* we will skip it and 16 more for good luck.


slSAVE_LOGO	GOSUBL	srRCL9_A	PICT GROB data @
	LCHEX	#00045	offset to field
	C=C+A	A
	D0=C
	GOSUBL	srRCL8_A	STACK GROB data @
	LCHEX	#005B0	offset
	C=C+A	A
	D1=C

	LCHEX	#00011	17+1 lines

slGET_LINE	A=DAT0	12	get 12 nibbles from screen
	DAT1=A	12	save them here
	D0=D0+	12	inc 12 nibbles
	D1=D1+	12	ditto
	A=DAT0	12	get 12 more
	DAT1=A	12	save 12 more
	D0=D0+	12	you know.
	D1=D1+	12	ditto
	D0=D0+	10	move to next PICT line

	C=C-1	A
	GONC	slGET_LINE	do next line
	RTN


dhHIGH	GOSUBL	cfCLEAR
	C=PC
	GOTO	dhNEXT

	NIBHEX	849474840235	| "HIGH SCORES"
	NIBHEX	34F425543500	|
	NIBHEX	13020200	 "1  "
	NIBHEX	23020200	 "2  "
	NIBHEX	33020200	 "3  "
	NIBHEX	43020200	 "4  "
	NIBHEX	53020200	 "5  "


dhNEXT	C=C+CON	A,4
	D0=C
	LCHEX	#00225	{#19h #4h}
	GOSUB	fdFONT	display "HIGH SCORES"
	LCHEX	#0094C	{#16h #12h}
	D=C	A	save it here

	C=0	A
	GOSUBL	srSTO38_C	marker

	GOSUBL	srRCL32_C	high score data @
	R4=C.F	A

dhNEXT_LINE	C=D	A	get coords
	GOSUB	fdFONT	display a number
	AD0EX		save D0 here
	B=C	A	save C here
	C=R4.F	A	get next high score data @
	D0=C
	C=DAT0	A	read a score
	D0=D0+	5	move to initials
	CD0EX		score into D0, @ into C
	R4=C.F	A	put address here
	C=B	A	get coord back
	GOSUB	ndNUMBER	display the score
	C=C+CON	A,5	move over 5 pixels
	B=C	A	save coords again
	C=R4.F	A	@ of initials
	D0=C

	C=DAT0	2	get one letter
	?CBIT=0	7	this means we don't have 'FF' (not new hs)
	GOYES	dhNL_NEXT1
	C=B	A	get coords
	GOSUBL	srSTO38_C	store it here (and mark)
	LCHEX	#00202020	3 spaces and a NULL
	DAT0=C	8	clear out bogus initials
	CD0EX		get @ of bogus initials
	GOSUBL	srSTO37_C	put it here
	D0=C
	D0=D0+	8	point past bogus initials (02020200)
	GOTO	dhNL_NEXT2	don't display

dhNL_NEXT1	C=B	A	get coords back
	GOSUB	fdFONT	display initials
dhNL_NEXT2	CD0EX		get D0 (which now points to next score)
	R4=C.F	A	save it here

	D0=A	A	restore pointer into dh data

	LCHEX	#00395	7 lines down
	C=C+D	A
	D=C	A
	LCHEX	#01B35	stopping point
	?C=D	A	still more?
	GOYES	dhNO_MORE
	GOTO	dhNEXT_LINE

dhNO_MORE	GOSUBL	srRCL38_C	check marker
	?C#0	A	A new high score?
	GOYES	dhSOUND
	GOTO	dhCHECK_KEY

dhSOUND	GOSUBL	soHIGH_SCORE
	GOSUBL	srRCL38_C	coords
	GOLONG	ilINPUT_LET	get initials
dhIL_RTN	R2=C		letters returned in C, put them in R2
	GOSUBL	srRCL37_C	get back @
	D0=C		this is @ to store initials into
	D0=D0+	4	move to last letter

	LCHEX	#00002	2+1 times through loop
	B=C	A
	A=R2		get letters
dhNS_NEXT	LCHEX	#000FF	mask
	C=C&A	A	get letter
	?C#0	A	not a zero?
	GOYES	dhNS_WRITE
	LCHEX	#20	a space
dhNS_WRITE	DAT0=C	2	write it out
	ASR	W	shift A right 1 nibble
	ASR	W	shift another nibble
	D0=D0-	2	back D0 up to next letter
	B=B-1	A
	GONC	dhNS_NEXT

dhCHECK_KEY	LCHEX	#1FF	| wait here until
	OUT=C		| they stop pressing
	GOSBVL	=CINRTN	| a key
	LAHEX	#0003F	|
	A=A&C	A	|
	?A#0	A	|
	GOYES	dhCHECK_KEY	|

	LCHEX	#00800	about 10 seconds
	GOSUB	waWAIT
	GOLONG	mnMENU	Always return to Menu

* waWAIT delays, also waiting for a keypress.  Load C, field A with
* the amount to wait (000CC is about 1 second) and call waWAIT.
* If you press a key before the time is up, then the delay stops.
* If you press [S], then the sound is toggled, (and the delay
* stops)  If you press [ON] you are immediately returned to RPL.

waWAIT	B=C	A
waNEXT	LCHEX	#00013	a short delay
	GOSUBL	udDELAY

	LCHEX	#1FF
	OUT=C
	GOSBVL	=CINRTN
	LAHEX	#0003F	all keys
	A=A&C	A
	?A#0	A	have you pressed a key?
	RTNYES		yes? get out

	B=B-1	A
	GONC	waNEXT
	RTN

* DIV divides A, field W by C, field W.  The result is placed in
* field W of both registers A and C, and the remainder
* in B, field W.

DIV	P=	0
D_AGAIN	?C#0	S
	GOYES	D_NOTBIG
	?C>=A	W
	GOYES	D_NOTBIG
	CSL	W
	P=P+1
	GONC	D_AGAIN

D_NOTBIG	B=A	W
	A=0	W
	A=A-1	P
D_ADD	A=A+1	P
D_SUB	B=B-C	W
	GONC	D_ADD
	B=B+C	W
	P=P-1
	GOC	D_DONE
	CSR	W
	GONC	D_SUB
D_DONE	C=A	W
	P=	0
	RTNCC
ENDCODE
	TURNMENUON
	RECLAIMDISP
	FIFTYSIX
	SWAP
	ITE
	SetSysFlag
	ClrSysFlag
	2DROP
	'
	ID DIAM.HIGH
	XEQSTOID
	DROP
;
